<?php

include ES_PLUGIN_PATH . '/admin/classes/import/es-data-provider-interface.php';
include ES_PLUGIN_PATH . '/admin/classes/import/class-data-provider.php';
include ES_PLUGIN_PATH . '/admin/classes/import/class-csv-data-provider.php';

/**
 * Class Es_Import
 */
class Es_Import
{
    /**
     * @var array
     */
    protected $_file = null;
    /**
     * @var Es_Messenger_Interface
     */
    protected $_logger;
    /**
     * @var array
     */
    protected $_options = array();

    /**
     * @return Es_Messenger|Es_Messenger_Interface
     */
    public function get_logger()
    {
        return $this->_logger;
    }

    /**
     * Es_Import constructor.
     *
     * @param $file
     * @param array $options
     * @throws Exception
     */
    public function __construct( $file = null, array $options = array() )
    {
        if ( ! $file || ( is_string( $file ) && ! file_exists( $file ) ) ) {
            throw new Exception( __( 'Source file can\'t to be empty.', 'es-plugin' ) );
        }

        if ( ! empty( $options['logger'] ) && $options['logger'] instanceof Es_Messenger_Interface ) {
            $this->_logger = $options['logger'];
        } else {
            $this->_logger = new Es_Messenger( 'import_messages' );
            $this->_logger->clean_container();
        }

        $this->_options = $options;

        $this->_file = $file;
    }

    /**
     * Return num of items.
     *
     * @return mixed
     */
    public function get_num_items()
    {
        $count = $this->get_data_provider() ? $this->get_data_provider()->get_count() : 0;
        $skip = $this->get_option( 'skip_first', 0 );

        return $skip && $count ? $count - 1 : $count;
    }

    /**
     * Prepare object for import.
     *
     * @return $this
     */
    public function prepare() {
        if ( is_array( $this->_file ) ) {
            $this->_file = $this->upload_file();
        }
        return $this;
    }

    /**
     * Return path to the source file.
     *
     * @return array|null|string
     */
    public function get_file()
    {
        return $this->_file;
    }

    /**
     * Return class names of data providers.
     *
     * @return array
     */
    public function get_data_providers()
    {
        return apply_filters( 'es_import_data_providers', array(
            'csv' => 'Es_Csv_Data_Provider',
        ) );
    }

    /**
     * Upload file.
     *
     * @return mixed
     */
    public function upload_file()
    {
        // Include wp library for uploading filess
        if ( ! function_exists( 'wp_handle_upload' ) ) {
            $path = ABSPATH . 'wp-admin/includes/file.php';
            require_once( $path );
        }

        // Upload file.
        $file = wp_handle_upload( $this->_file, array( 'test_form' => false ) );

        if ( $file && ! isset( $file['error'] ) ) {
            return $file['file'];
        } else {
            return false;
        }
    }

    /**
     * Return object of data provider.
     *
     * @return Es_Data_Provider_Interface
     * @throws Exception
     */
    public function get_data_provider()
    {
        if ( ! $this->_file ) {
            throw new Exception( __( 'Can\'t return a data provider. Source file is empty.', 'es-plugin' ) );
        }

        // Get list of data providers.
        $data_providers = $this->get_data_providers();
        // Get file extension.
        $ext = pathinfo( $this->_file, PATHINFO_EXTENSION );
        // Get data provider class name.
        $class_name = ! empty( $data_providers[ $ext ] ) ? $data_providers[ $ext ] : null;
        // Get data provider object.
        $data_provider = class_exists( $class_name ) ? new $class_name( $this->_file, $this->_options ) : null;

        return $data_provider instanceof Es_Data_Provider_Interface ? $data_provider : null;
    }

    /**
     * Return import option by name.
     *
     * @param $name
     * @param $default
     * @return mixed
     */
    public function get_option( $name, $default ) {
        return ! empty( $this->_options[ $name ] ) ? $this->_options[ $name ] : $default;
    }

    /**
     * Import new property using index.
     *
     * @param int $index
     * @return bool|Es_Property
     */
    public function one( $index = 0 )
    {
        // Get data provider.
        $data_provider = $this->get_data_provider();

        // If data provider is correct.
        if ( $data_provider instanceof Es_Data_Provider_Interface ) {

            // Get property from file using line number.
            $item = $data_provider->get_item( $index );

            // If item is correct
            if ( is_array( $item ) && ! empty( $item ) ) {

                // Get all defined import fields.
                $import_fields = $this->get_import_fields();

                // Generate default post array.
                $post_arr = apply_filters( 'es_import_property_default_array', array(
                    'post_status' => $this->get_option( 'post_status', 'publish' ),
                    'post_type' => Es_Property::get_post_type_name(),
                    'post_author' => get_current_user_id(),
                    'post_content' => ' ',
                ), $item, $data_provider, $index );

                // Set line column number.
                $i = 0;

                $entity_fields = $this->get_fields();

                if ( $entity_fields && $import_fields ) {
                    foreach ( $import_fields as $field ) {
                        if ( ! empty( $entity_fields[ $field ] ) ) {
                            $options = $entity_fields[ $field ];
                            if ( ! empty( $options['type'] ) && $options['type'] == 'system' ) {
                                $post_arr[ $field ] = isset( $item[ $i ] ) ? $item[ $i ] : null;
                            }
                        }

                        $i++;
                    }
                }

                if ( ! empty( $post_arr['post_title'] ) && empty( $post_arr['post_name'] ) ) {
                    $post_arr['post_name'] = sanitize_title( $post_arr['post_title'] );
                }

	            if ( empty( $post_arr['post_content'] ) ) {
		            $post_arr['post_content'] = '';
	            }

                // Apply filter for generated property array.
                $post_arr = apply_filters(
                    'es_import_property_default_array_generated',
                    $post_arr,
                    $item,
                    $data_provider,
                    $index
                );

                // Insert post.
                $post_id = wp_insert_post( $post_arr );
                $meta = array();
                $custom = array();

                if ( ! $post_id instanceof WP_Error ) {
                    $i = 0;
                    foreach ( $import_fields as $field ) {
                        $options = $entity_fields[ $field ];

                        if ( in_array( $field, $import_fields ) ) {

                            switch ( $options['type'] ) {

                                case 'meta':
                                    $meta[ $field ] = $item[ $i ];

                                    break;

                                case 'file':
                                    $file['name'] = basename( $item[$i] );
                                    $file['tmp_name'] = download_url( $item[$i] );

                                    if ( ! is_wp_error( $file['tmp_name'] ) ) {
                                        $attachmentId = media_handle_sideload( $file, $_POST['post_id'] );

                                        if ( $attachmentId ) {
                                            $meta[ $field ] = $attachmentId;
                                        }

                                    } else {
                                        if ( $file['tmp_name'] instanceof WP_Error ) {
                                            $this->_logger->set_message( sprintf( __( 'File of %s field didn\'t upload', 'es-plugin' ), $field ) . ': ' . $file['tmp_name']->get_error_message(), 'warning' );
                                        }
                                    }

                                    break;

                                case 'gallery':
                                    $file = array();

                                    if ( empty( $item[$i] ) ) break;

                                    $file['name'] = basename( $item[$i] );
                                    $file['tmp_name'] = download_url( $item[$i] );

                                    if ( ! is_wp_error( $file['tmp_name'] ) ) {
                                        $attachmentId = media_handle_sideload($file, $_POST['post_id']);

                                        if ( $attachmentId ) {
                                            $meta[ $field ][] = $attachmentId;
                                        }

                                    } else {
                                        if ( $file['tmp_name'] instanceof WP_Error ) {
                                            $this->_logger->set_message( __( 'Image field', 'es-plugin' ) . ': ' . $file['tmp_name']->get_error_message(), 'warning' );
                                        }
                                    }

                                    break;

                                case 'custom':
                                    $value = $item[ $i ];

                                    if ( ! empty( $value ) ) {
                                        $value = explode( ':', $value );
                                        if ( ! empty( $value[0] ) && ! empty( $value[1] ) ) {
                                            $custom[ $value[0] ] = $value[1];
                                        }
                                    }
                                    break;

                                case 'taxonomy':
                                    wp_set_post_terms( $post_id, $item[ $i ], $field );

                                    break;

                                case 'system':
                                    if ( ! empty( $options['system_type'] ) && $options['system_type'] == 'taxonomy' ) {
                                        wp_set_post_terms( $post_id, $item[ $i ], $field );
                                    }

                                    break;
                            }
                        }
                        $i++;
                    }

                    if ( ! empty( $meta['latitude'] ) && ! empty( $meta['longitude'] ) ) {
                        $components = ES_Address_Components::get_google_components( array(
                            $meta['latitude'], $meta['longitude']
                        ) );
                    } else if ( ! empty( $meta['address'] ) ) {
                        $components = ES_Address_Components::get_google_components( $meta['address'] );
                    } else {
                        $components = null;
                    }

                    if ( ! empty( $components ) ) {
                        if ( ! empty( $components->status ) && $components->status == 'OK' ) {
                            $res = $components->results[0];
                            $meta['address'] = $res->formatted_address;

                            if ( empty( $meta['latitude'] ) && empty( $meta['longitude'] ) ) {
                                $meta['latitude'] = $res->geometry->location->lat;
                                $meta['longitude'] = $res->geometry->location->lng;
                            }

                            if ( ! empty( $res->address_components ) ) {
                                $meta['address_components'] = json_encode( $res->address_components );
                            }
                        } else {
                            $this->_logger->set_message( __( 'Incorrect property address string or coordinates.', 'es-plugin' ), 'warning' );
                        }
                    }

                    $property = es_get_property( $post_id );

                    if ( $meta ) {
                        $property->save_fields( $meta );
                    }

                    if ( $custom ) {
                        $property->save_custom_data( $custom );
                    }

                    $id = ! empty( $post_arr['post_title'] ) ? $post_arr['post_title'] : '#' . $post_id;

                    $this->_logger->set_message( __( 'Property ' . $id . ' created!', 'es-plugin' ), 'success' );
                } else {
                    $error = $post_id instanceof WP_Error ? $post_id->get_error_message() : null;
                    $this->_logger->set_message( __( 'Post isn\'t created!', 'es-plugin' ) . $error, 'error' );
                }
            }
        }

        return ! empty( $post_id ) && is_int( $post_id ) ? es_get_property( $post_id ) : false;
    }

    /**
     * Return concrete fields for import.
     *
     * @return array
     */
    public static function get_import_fields()
    {
        return get_option( 'es_import_fields', array_keys( static::get_fields() ) );
    }

    /**
     * Return all fields for import properties.
     *
     * @return mixed
     */
    public static function get_fields()
    {
        $property = es_get_property( null );
        $fields = $property::get_fields();

        foreach ( $fields as $key => $field ) {
            if ( empty( $field['import_support'] ) ) {
                unset( $fields[ $key ] );
            } else {
                if ( ! empty( $field['system'] ) ) {
                    $fields[ $key ]['type'] = 'system';
                } else if ( ! empty( $field['system_type'] ) && $field['system_type'] == 'taxonomy' ) {
                    $fields[$key]['type'] = 'taxonomy';
                } else if ( ! empty( $field['gallery'] ) ) {
                    $fields[ $key ]['type'] = 'gallery';
                } else if ( $field['type'] != 'custom' && $field['type'] != 'gallery' && $field['type'] != 'file' ) {
                    $fields[ $key ]['type'] = 'meta';
                }
            }
        }

        return apply_filters( 'es_import_get_fields', $fields );
    }
}
