(function() {
    'use strict';

    /**
     * Google map class.
     *
     * @param selector
     * @param lon
     * @param lat
     * @param zoom
     * @returns {EsGoogleMap}
     */
    var EsGoogleMap = function(selector, lon, lat, zoom) {
        this.selector = selector;
        this.zoom = parseFloat( zoom ) || null;
        this.lon = parseFloat(lon);
        this.lat = parseFloat(lat);
        this.markers = [];

        /**
         * Initialize google maps method.
         * @return object
         */
        this.init = function() {

            var options = {};

            if ( this.zoom ) {
                options.zoom = this.zoom;
            }

            if (this.lon && this.lat) {
                options.center = {lat: this.lat, lng: this.lon};
                options.zoom = options.zoom ? options.zoom : 16;
            }

            this.instance = new google.maps.Map(this.selector, options);

            google.maps.event.trigger(this.selector, 'resize');

            return this;
        };

        /**
         *
         * Set marker to the map.
         *
         * @param position
         * @returns {EsGoogleMap}
         */
        this.setMarker = function(position) {
            position = position || {lat: this.lat, lng: this.lon};

            var marker = new google.maps.Marker({
                position: position,
                map: this.instance
            });

            this.markers.push(marker);

            return marker;
        };

        /**
         * Custom marker class.
         *
         * @param map
         * @param content
         * @param position
         * @param args
         * @constructor
         */
        function CustomMarker(map, content, position, args) {
            this.content = content;
            this.position = position;
            this.args = args;
            this.setMap(map);
        }

        /**
         * Call overlay prototype.
         * @type {google.maps.OverlayView}
         */
        CustomMarker.prototype = new google.maps.OverlayView();

        /**
         * Draw marker.
         *
         * @return void
         */
        CustomMarker.prototype.draw = function() {

            var self = this;

            var div = this.div;

            if (!div) {

                div = this.div = document.createElement('div');

                div.className = 'es-marker';
                div.innerHTML = this.content;

                google.maps.event.addDomListener(div, "click", function(event) {
                    google.maps.event.trigger(self, "click");
                });

                var panes = this.getPanes();
                panes.overlayImage.appendChild(div);
            }

            var point = this.getProjection().fromLatLngToDivPixel(this.position);

            if (point) {
                div.style.left = point.x + 'px';
                div.style.top = point.y + 'px';
            }
        };

        CustomMarker.prototype.remove = function() {
            if (this.div) {
                this.div.parentNode.removeChild(this.div);
                this.div = null;
            }
        };

        CustomMarker.prototype.getPosition = function() {
            return this.position;
        };

        /**
         *
         * Set marker to the map.
         *
         * @param content
         * @param position
         * @returns {EsGoogleMap}
         */
        this.setCustomMarker = function(content, position) {
            var marker = function() {};
            var instance = this.instance;
            position = position || {lat: this.lat, lng: this.lon};

            position = new google.maps.LatLng(position.lat, position.lng);

            return new CustomMarker(this.instance, content, position);
        };

        /**
         *
         * Get address info using coordinates.
         *
         * @param lat
         * @param lon
         * @param callback
         */
        this.getGeocoderInfo = function(lat, lon, callback) {
            var geocoder = new google.maps.Geocoder();
            var latLon = new google.maps.LatLng(lat, lon);

            geocoder.geocode({
                latLng: latLon
            }, callback);
        };

        return this;
    };

    window.EsGoogleMap = EsGoogleMap;
})();
