<?php

/**
 * Class Es_Agent
 *
 * @property string $company
 * @property int $property_qty
 * @property int $rating
 * @property int $name
 * @property int $status
 * @property int $tel
 * @property int $disabled
 * @property string $about
 * @property array $unpublished_properties
 *
 * Subscription fields appears when subscription functional is enabled.
 *
 * @property int $subscription_property_qty
 * @property bool $subscription_active
 * @property int $subscription_featured_qty
 * @property int $subscription_order_id
 * @property int $subscription_id
 * @property int $subscription_published_posts
 * @property int $subscription_expired
 * @property int $subscription_featured_posts
 *
 */
class Es_Agent extends Es_User
{
    const STATUS_ACTIVE = 1;
    const STATUS_DISABLED = 0;

    /**
     * Return agent full name.
     *
     * @return null|string
     */
    public function get_full_name()
    {
        return $this->name;
    }

    /**
     * Return entity prefix string.
     *
     * @return string
     */
    public function get_entity_prefix()
    {
        return apply_filters( 'es_agent_entity_prefix', 'es_agent_' );
    }

    /**
     * Initialize agent filters and actions.
     *
     * @return void
     */
    public static function init()
    {
        add_filter( 'get_avatar' , array( __CLASS__, 'get_avatar' ) , 1 , 5 );
	}

    /**
     * Customize avatar for agents.
     *
     * @param $avatar
     * @param $id_or_email
     * @param $size
     * @param $default
     * @param $alt
     * @return string
     */
    public static function get_avatar( $avatar, $id_or_email, $size, $default, $alt )
    {
        $user = false;

        if ( is_numeric( $id_or_email ) ) {

            $id = (int) $id_or_email;
            $user = get_user_by( 'id' , $id );

        } elseif ( is_object( $id_or_email ) ) {

            if ( ! empty( $id_or_email->user_id ) ) {
                $id = (int) $id_or_email->user_id;
                $user = get_user_by( 'id' , $id );
            }

        } else {
            $user = get_user_by( 'email', $id_or_email );
        }

        if ( $user && is_object( $user ) && in_array( static::get_role_name(), $user->roles ) ) {
            $agent = new self( $user->ID );

            global $post;

            $src = $agent->get_image_url( $size ) ? $agent->get_image_url( $size ) : ES_PLUGIN_URL . 'assets/images/agent.png';

            $avatar = "<img alt='{$alt}' src='{$src}' class='es-agent-avatar' width='{$size}' height='{$size}'/>";
        }

        return $avatar;
    }

    /**
     * Return agent role name.
     *
     * @return mixed
     */
    public static function get_role_name()
    {
        return apply_filters( 'es_agent_role_name', 'agent_role' );
    }

    /**
     * Return agent custom fields.
     *
     * @return mixed
     */
    public static function get_fields()
    {
        $fields = array(
            'company' => array(
                'label' => __( 'Company' , 'es-plugin' ),
            ),

            'rating' => array(
                'label' => __( 'Rating' , 'es-plugin' ),
            ),

            'name' => array(
                'label' => __( 'Name' , 'es-plugin' ),
            ),

            'property_qty' => array(
                'label' => __( 'Sold properties' , 'es-plugin'),
            ),

            'tel' => array(
                'label' => __( 'Telephone' , 'es-plugin'),
            ),

            'about' => array(
                'label' => __( 'About' , 'es-plugin'),
            ),

            'status' => array(
                'label' => __( 'Status' , 'es-plugin'),
            ),

            'profile_attachment_id' => array(
                'label' => __( 'Profile image' , 'es-plugin'),
            ),

            'unpublished_properties' => array(),
        );

        if ( es_subscription_is_enabled() ) {
            $fields = array_merge( $fields, array(
                'subscription_property_qty' => array(),
                'subscription_active' => array(),
                'subscription_featured_qty' => array(),
                'subscription_order_id' => array(),
                'subscription_id' => array(),
                'subscription_published_posts' => array(),
                'subscription_expired' => array(),
                'subscription_featured_posts' => array(),
            ) );
        }

        return apply_filters( 'es_agent_fields', $fields );
    }

    /**
     * Return list of all users companies.
     *
     * @return array
     */
    public static function get_companies_list()
    {
        global $wpdb;
        return $wpdb->get_col( "SELECT meta_value 
                                FROM " . $wpdb->usermeta . "
                                WHERE meta_key = 'es_agent_company' 
                                GROUP BY meta_value" );
    }

    /**
     * Return Agents rating list.
     *
     * @return array
     */
    public static function get_rating_list()
    {
        return apply_filters( 'es_get_agent_rating_list', array(
            1 => __( 'Bad', 'es-plugin' ),
            2 => __( 'Poor', 'es-plugin' ),
            3 => __( 'Regular', 'es-plugin' ),
            4 => __( 'Good', 'es-plugin' ),
            5 => __( 'Excellent', 'es-plugin' )
        ) );
    }

    /**
     * Return custom fields data.
     *
     * @return mixed
     */
    public function get_custom_data()
    {
        return get_user_meta( $this->_id, 'es_custom_data' );
    }

    /**
     * Return agent profile image URL.
     *
     * @param string $size
     * @return null
     */
    public function get_image_url( $size = 'thumbnail' )
    {
        $url = null;

        if ( $this->profile_attachment_id ) {
            $image = wp_get_attachment_image_src( $this->profile_attachment_id, $size, true );
            $url = ! empty( $image[0] ) ? $image[0] : null;
        }

        return apply_filters( 'es_agent_profile_image_url', $url, $this->profile_attachment_id, $this );
    }

    /**
     * Register new agent.
     *
     * @param array $data
     *
     * @return Es_Agent|WP_Error
     */
    public function register( $data )
    {
        if ( ! empty( $data['username'] ) && ! empty( $data['email'] ) ) {

            $data = apply_filters( 'es_agent_register_data', $data );

            add_action( 'user_register', function( $user_id ) {
                $user = get_user_by( 'ID', $user_id );
                $user->set_role( Es_Agent::get_role_name() );
            } );

            // Register new user.
            $user_id = register_new_user( $data['username'], $data['email'] );

            if ( ! is_wp_error( $user_id ) ) {
                // Set agent ID.
                $this->_id = $user_id;
                /** @var WP_User $user */
                $user = $this->get_entity();
                // Set User role as Agent.
                $user->set_role( static::get_role_name() );

                // Get all agent fields.
                $fields = static::get_fields();

                if ( ! empty( $_FILES['agent_photo'] ) ) {
                    if ( ! function_exists( 'wp_handle_upload' ) ) {
                        require_once( ABSPATH . 'wp-admin/includes/image.php' );
                        require_once( ABSPATH . 'wp-admin/includes/file.php' );
                        require_once( ABSPATH . 'wp-admin/includes/media.php' );
                    }

                    if ( $attachment_id = media_handle_sideload( $_FILES['agent_photo'], -1 ) ) {
                        if ( ! $attachment_id instanceof WP_Error ) {
                            $data['profile_attachment_id'] = $attachment_id;
                        }
                    }
                }

                // Save custom agent fields.
                foreach ( $data as $field => $value ) {
                    if ( $field == 'website' ) {
                        $update['user_url'] = $value;
                    }

                    if ( $field == 'about' ) {
                        $update['description'] = $value;
                    }

                    if ( ! empty( $fields[ $field ] ) ) {
                        $this->save_field_value( $field, $value );
                    }
                }

                // Update user description and URL.
                if ( ! empty( $update ) ) {
                    $update['ID'] = $this->_id;
                    $update = apply_filters( 'es_agent_user_data_update', $update );
                    wp_update_user( $update );
                }

                /**
                 * Agent after register action.
                 *
                 * @param Es_Agent $this
                 */
                do_action( 'es_agent_after_register', $this );

            } else {
                return $user_id;
            }
        } else {
            $error = new WP_Error();
            $error->add( 'error', __( 'Email or username is empty.', 'es-plugin' ) );

            return $error;
        }

        return $this;
    }

    /**
     * @return string
     */
    public function get_base_field_name()
    {
        return 'agent';
    }

    /**
     * Change agent status (active / disabled).
     *
     * @param $status
     */
    public function change_status( $status )
    {
        $old_status = $this->status;
        $this->save_field_value( 'status', $status );
        do_action( 'es_agent_change_status', $this, $status, $old_status );
    }

	/**
	 * @return bool
	 */
    public function is_agent() {

    	$user = $this->get_entity();
    	if ( in_array( static::get_role_name(), $user->roles ) ) {
    		return true;
	    }
	    return false;
    }
}
