<?php

// Include PDF generator library.
include ES_PLUGIN_PATH . '/libraries/dompdf/autoload.inc.php';

use Dompdf\Dompdf;
use Dompdf\Options;

/**
 * Class Es_Property_Pdf
 */
class Es_Property_Pdf
{
    /**
     * @var WP_Post
     */
    protected $_post;
    /**
     * @var Dompdf
     */
    protected $_tcpdf;

    /**
     * Es_Property_Pdf constructor.
     *
     * @param WP_Post $post
     */
    public function __construct( WP_Post $post )
    {
        add_action( 'es_single_description_tab_pdf', array( $this, 'single_description_tab' ) );
        add_action( 'es_single_features_pdf', array( $this, 'single_features' ) );
        add_action( 'es_single_info_tab_pdf', array( $this, 'single_info' ) );
        add_action( 'es_single_agent_tab_pdf', array( $this, 'agent_tab' ) );
        add_action( 'es_single_render_tab_pdf', array( $this, 'single_render_tab' ), 10, 2 );

        $this->_post = $post;
        $options = new Options();
        $options->set('isRemoteEnabled', true);
        $this->_tcpdf = new Dompdf( $options );
        $this->_tcpdf->setPaper('A4', 'portrait');
    }

    /**
     * PDF template path.
     *
     * @return mixed
     */
    public function get_tempalte_path()
    {
        return apply_filters( 'es_property_pdf_get_tempalte_path', ES_TEMPLATES . 'property/pdf.php' );
    }

    /**
     * Render pdf page.
     *
     * @return void
     */
    public function output()
    {
        $post = $this->_post;

        global $es_settings;
        $property = es_get_property($post->ID);

        ob_start();
        $path = apply_filters( 'es_property_pdf_template_path', $this->get_tempalte_path() );

        include( $path );

        $html = ob_get_clean();

        $this->_tcpdf->loadHtml($html);

        $this->_tcpdf->render();

        $this->_tcpdf->stream( 'property.pdf', array( 'Attachment' => 0 ) );
    }

    /**
     * Return fields for render.
     *
     * @return mixed
     */
    public static function get_single_fields_data()
    {
        $data = Es_Property_Single_Page::get_single_fields_data();

        return apply_filters( 'es_single_pdf_fields_data', $data );
    }

    /**
     * Render custom section action.
     *
     * @param $id integer
     *    Section ID.
     * @param $section array
     *    Section config array.
     */
    public function single_render_tab( $id, $section )
    {
        if ( ! empty( $section['fbuilder'] ) ) {
            $fields = Es_FBuilder_Helper::get_entity_fields( 'property', $id );
            $data = Es_Property_Single_Page::get_fields_render_data( $fields );

            if ( $data ) : ?>
                <div class="container">
                    <h2><?php echo $section['label']; ?></h2>
                    <?php do_action( 'es_single_tabbed_content_after_pdf', $id ); ?>
                </div>
            <?php endif;
        }
    }

    /**
     * Return featured data.
     *
     * @return mixed
     */
    public static function get_features_data()
    {
        $data = Es_Property_Single_Page::get_features_data();

        return apply_filters( 'es_single_pdf_features_data', $data );
    }

    /**
     * Return sections.
     *
     * @return mixed
     */
    public static function get_sections() {
        $sections = Es_Property_Single_Page::get_sections();

        return apply_filters( 'es_property_pdf_get_sections', $sections );
    }

    /**
     * Render property description tab content.
     *
     * @return void
     */
    function single_description_tab() {
        global $post;
        if ( $post->post_excerpt ) : ?>
            <div class="container">
                <h2><?php echo __( 'Description', 'es-plugin' ); ?></h2>
                <?php the_excerpt(); ?>
                <?php do_action( 'es_single_tabbed_content_after_pdf', 'es-description' ); ?>
            </div>
        <?php endif;
    }

    /**
     * Render basic info container.
     *
     * @return void
     */
    public function single_info() {
        ?>
        <div class="container">
            <h2><?php _e( 'Basic Facts', 'es-plugin' ); ?></h2>
            <?php if ( $fields = Es_Property_Pdf::get_single_fields_data() ) : $i = 2; ?>
            <table cellpadding="0">
                <?php foreach ( $fields as $name => $value ) : if ( empty( $value[ key( $value ) ] ) ) continue; ?>
                    <?php echo ! ( $i % 2 ) ? '<tr>' : null; ?>
                    <td><?php echo key( $value ); ?>: <?php echo $value[ key( $value ) ]; ?></td>
                    <?php echo ( $i % 2 ) ? '</tr>' : null; ?>
                <?php $i++; endforeach; ?>

                <?php echo ! ( $i % 2 ) ? '<td></td></tr>' : null; ?>

            </table>
            <?php endif; ?>
        </div>
    <?php }

    /**
     * Render features list.
     *
     * @return void
     */
    public function single_features() {
        if ( es_get_the_features() || es_get_the_amenities() ) : $fields = self::get_features_data(); ?>

        <div class="container container-features">
            <table>
                <tr>
                    <?php foreach ( $fields as $name => $items ): ?>
                        <td>
                            <h2><?php echo $name; ?></h2>
                            <?php if ( ! empty( $items ) ) : ?>
                                <ul>
                                <?php foreach ( $items as $item ) : ?>
                                    <li>
                                        <img src="<?php echo ES_PLUGIN_PATH . '/assets/images/es_feature_tick_pdf.png'; ?>" alt="#"/>
                                        <?php echo $item->name; ?>
                                    </li>
                                <?php endforeach; ?>
                                </ul>
                            <?php endif; ?>
                        </td>
                    <?php endforeach; ?>
                </tr>

            <?php do_action( 'es_single_tabbed_content_after_pdf', 'es-features' ); ?>
            </table>
        </div>
        <?php endif;
    }

    /**
     * Render agent container.
     *
     * @return void
     */
    public function agent_tab() {
        global $es_settings;
        $property = es_get_property( $this->_post->ID );

        if ( ( $agent = $property->get_agent() ) && $es_settings->show_agent_tab ) : $user = $agent->get_entity(); ?>
            <div class="container container-agent">
                <table>
                    <tr>
                        <td class="agent-image">
                            <?php if ( $image = $agent->get_image_url( 'es-agent-size' ) ): ?>
                                <img src="<?php echo $image; ?>" alt="<?php _e( 'Agent image', 'es-plugin' ); ?>">
                            <?php elseif ( $image = es_get_default_thumbnail( 'es-agent-size', 'agent' ) ) : ?>
                                <?php echo $image; ?>
                            <?php endif; ?>
                        </td>
                        <td class="agent-content">
                            <table>
                                <tr>
                                    <td class="agent-info">
                                        <span class="es-agent__field es-field__name"><?php echo $agent->get_full_name(); ?></span>
                                        <span class="es-agent__field es-field__tel"><?php echo $agent->tel; ?></span>

                                        <?php if ( $email = $user->user_email ) : ?>
                                            <span class="es-agent__field">
                                                <?php echo $email ?>
                                            </span>
                                        <?php endif; ?>

                                        <?php if ( $url = $user->user_url ) : ?>
                                            <span class="es-agent__field">
                                                <a href="<?php echo $url; ?>"><?php echo $url; ?></a>
                                            </span>
                                        <?php endif; ?>

                                        <?php if ($custom = $agent->get_custom_data()) : ?>
                                            <?php foreach ($custom as $key => $value) : ?>
                                                <span class="es-agent__field"><?php echo key($value); ?>
                                                    : <?php echo $value[key($value)]; ?></span>
                                            <?php endforeach; ?>
                                        <?php endif; ?>

                                        <?php if ($agent->company) : ?>
                                            <span class="es-agent__field es-field__company"><?php _e('Company', 'es-plugin'); ?>
                                                : <?php echo $agent->company; ?></span>
                                        <?php endif; ?>
                                    </td>
                                    <td class="agent-rating">
                                        <?php if ( $agent->rating ) : ?>
                                            <div class="rating-wrap">
                                                <span class="rating-label"><?php _e( 'Rating', 'es-plugin' ); ?>:</span>
                                                <?php es_render_rating( $agent->rating ); ?>
                                            </div>
                                        <?php endif; ?>
                                    </td>
                                </tr>
                            </table>
                            <div class="agent-description">
                                <?php echo $user->description; ?>
                            </div>
                        </td>
                    </tr>
                </table>
            </div>
        <?php endif;
    }
}
