<?php

/**
 * Class Es_Property
 *
 * @property string $address
 * @property string $video
 * @property float $price
 * @property bool $call_for_price
 * @property bool $bedrooms
 * @property bool $bathrooms
 * @property bool $floors
 * @property array $gallery
 * @property float $lot_size
 * @property float $area
 * @property float $year_built
 * @property float $latitude
 * @property float $longitude
 * @property int $featured
 * @property string $address_components
 */
class Es_Property extends Es_Post
{
    /**
     * @inheritdoc
     */
    public function get_entity_prefix()
    {
        return apply_filters('es_property_entity_prefix', 'es_property_');
    }

    /**
     * Save property address components.
     *
     * @param $data
     */
    public function save_address_components( $data )
    {
        if ( ! empty( $data ) ) {
            foreach ($data as $item) {
                $component_id = ES_Address_Components::save_component( $item );
                if ( ! empty( $component_id ) ) {
                    ES_Address_Components::save_property_component( $this->getID(), $component_id );
                }
            }
        }
    }

    /**
     * Return custom fields data.
     *
     * @return mixed
     */
    public function get_custom_data()
    {
        return get_post_meta( $this->_id, 'es_custom_data' );
    }

    /**
     * Save property fields.
     *
     * @param $data
     */
    public function save_fields( $data )
    {
        if ( ! empty( $data ) ) {
            $units = array();
            $fields = static::get_fields();
            $data = apply_filters( 'es_before_save_property_data', $data, $this );

            // Save address components.
            if ( ! empty($data['address_components'] ) ) {
                $this->save_address_components( json_decode( $data['address_components'] ) );
            }

            // Save another fields.
            foreach ( $fields as $key => $field ) {
                $value = isset( $data[ $key ] ) ? $data[ $key ] : null;

                $value = $key == 'call_for_price' && ! $value ? 0 : $value;
                $value = $key == 'video' ? esc_attr( $value ) : $value;

                if ( ! empty( $field['type'] ) && $field['type'] == 'file' ) {
                    $file_arr = array();

                    if ( ! empty( $_FILES[ $this->get_base_field_name() ]['name'][ $key ] ) ) {
                        foreach ( $_FILES[ $this->get_base_field_name() ] as $file_key => $field_value ) {
                            if ( ! empty( $field_value[ $key ] ) ) {
                                $file_arr[ $file_key ] = $field_value[ $key ];
                            }
                        }

                        if ( ! empty( $file_arr ) ) {
                            $value = media_handle_sideload( $file_arr, $this->getID() );

                            if ( ! is_wp_error( $value ) && ! empty( $this->{$key} ) ) {
                                wp_delete_attachment( $this->{$key}, true );
                            }
                        }
                    } else {
                        if ( ! isset( $data[ $key ] ) ) {
                            $value = '';
                            if ( ! empty( $this->{$key} ) ) {
                                wp_delete_attachment( $this->{$key}, true );
                            }
                        } else {
                            if ( isset( $data[ $key ] ) ) {
                                $value = $data[ $key ];
                            } else {
                                $value = $this->{$key};
                            }
                        }
                    }
                }

                $this->save_field_value( $key, $value );

                if ( ! empty( $field['units'] ) && ! empty( $value ) ) {
                   $units[ $key ] = array(
                       'units' => $fields[ $field['units'] ]['values'],
                       'value' => $value,
                       'unit' => $data[ $field['units'] ],
                   );
                }
            }

            if ( ! empty( $data['gallery'] ) ) {
                foreach ( $data['gallery'] as $attachment_id ) {
                    $attachment = get_post( $attachment_id );
                    if ( empty( $attachment->post_parent ) ) {
                        wp_update_post( array(
                            'ID' => $attachment_id,
                            'post_parent' => $this->getID(),
                        ) );
                    }
                }
            }

            if ( ! empty( $units ) ) {
                $this->save_units( $units );
            }
        }
    }

    /**
     * @param $units
     */
    public function save_units( array $units )
    {
        if ( ! empty( $units ) ) {
            foreach ( $units as $field => $item ) {
                if ( empty( $item['units'] ) ) continue;

                foreach ( $item['units'] as $unit => $label ) {
                    if ( $item['unit'] == $unit ) {
                        $value = $item['value'];
                    } else {
                        $func = apply_filters( 'es_prepare_unit_callback', 'es_prepare_unit', $unit, $item, $units, $this );
                        if ( function_exists( $func ) ) {
                            $value = call_user_func( $func, $item['unit'], $unit, $item['value'] );
                        }
                    }

                    if ( ! empty( $unit ) && ! empty( $value ) ) {
                        $this->save_field_value( $field . '_' . $unit, $value );
                    }
                }
            }
        }
    }

	/**
	 * Return property agent.
	 *
	 * @return Es_Agent|null
	 */
	public function get_agent()
	{
		$author = $this->get_entity()->post_author;
		$user = get_user_by( 'id', $author );

        if ( ! empty( $user->roles ) ) {
            $role = Es_Agent::get_role_name();

            if ( in_array( $role, $user->roles  ) ) {
                return es_get_agent( $user->ID );
            }
        }

		return null;
	}

    /**
     * Save custom property fields.
     *
     * @param $data
     */
    public function save_custom_data( $data )
    {
	    delete_post_meta( $this->getID(), 'es_custom_data' );

        if ( ! empty( $data ) ) {
            foreach ( $data as $key => $value ) {
                if ( ! empty( $key ) && ! empty( $value ) ) {
                    add_post_meta( $this->getID(), 'es_custom_data', array( $key => $value ), false );
                }
            }
        }
    }

    /**
     * @return mixed
     */
    public static function get_fields()
    {
        /** @var $es_settings Es_Settings_Container */
        /** @var $session_storage Es_Session_Storage */
        global $es_settings, $session_storage;

        $fields = $session_storage->get( 'property_fields' );
        $builder_fields = $session_storage->get( 'property_builder_fields' );

        if ( ! $builder_fields ) {
            $builder_fields = Es_FBuilder_Helper::get_fields( 'property' );
        }

        if ( ! $fields ) {

            $fields = array(

                'ID' => array(
                    'type' => 'text',
                    'system' => true,
                    'label' => __( 'Property ID', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'options' => array(
                        'readonly' => true,
                    )
                ),

                'post_author' => array(
                    'type' => 'list',
                    'system' => true,
                    'label' => __( 'Agent', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'values_callback' => array( 'es_get_users_list', array( 'role' => Es_Agent::get_role_name() ) ),
                    'visible' => current_user_can( 'manage_options' ) ? true : false,
                    'prompt' => __( '-- Choose agent --', 'es-plugin' ),
                    'options' => array(
                        'class' => 'es-select2',

                        'data-placeholder' => __( 'Agent', 'es-plugin' ),
                    ),
                ),

                'post_title' => array(
                    'type' => 'text',
                    'label' => __( 'Title', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'system' => true,
                    'import_support' => true,
                ),

                'date_added' => array(
                    'section' => 'es-info',
                    'label' => __( 'Date added', 'es-plugin' ),
                ),

                'price' => array(
                    'type' => 'number',
                    'tab' => 'es-basic-info',
                    'label' => __( 'Price', 'es-plugin' ),
                    'search_range_mode' => true,
                    'formatter' => 'area',
                    'import_support' => true,
                ),

                'price_note' => array(
	                'type' => 'text',
	                'tab' => 'es-basic-info',
	                'label' => __('Price note', 'es-plugin'),
                ),

                'call_for_price' => array(
                    'type' => 'checkbox',
                    'tab' => 'es-basic-info',
                    'label' => __( 'Call for price', 'es-plugin' ),
                    'options' => array( 'value' => 1, 'class' => 'es-switch-input' ),
                    'import_support' => true,
                ),

                'bedrooms' => array(
                    'type' => 'number',
                    'label' => __( 'Bedrooms', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'search_range_mode' => true,
                    'section' => 'es-info',
                    'import_support' => true,
                ),

                'bathrooms' => array(
                    'type' => 'number',
                    'label' => __( 'Bathrooms', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'search_range_mode' => true,
                    'options' => array( 'step' => 0.5 ),
                    'section' => 'es-info',
                    'import_support' => true,
                ),

                'floors' => array(
                    'type' => 'number',
                    'label' => __( 'Floors', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'search_range_mode' => true,
                    'section' => 'es-info',
                    'import_support' => true,
                ),

                'area' => array(
                    'type' => 'number',
                    'label' => __( 'Area', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'search_range_mode' => true,
                    'units' => 'area_unit',
                    'options' => array( 'step' => 0.1 ),
                    'formatter' => 'area',
                    'loop_callback' => array(
                        'callback' => 'es_the_formatted_area',
                        'args' => array( '', ' ', false ),
                    ),
                    'section' => 'es-info',
                    'import_support' => true,
                ),

                'area_unit' => array(
                    'type' => 'list',
                    'values' => $es_settings::get_setting_values( 'unit' ),
                    'template' => true,
                    'label' => false,
                    'skip_search' => true,
	                'default_value' => $es_settings->unit,
                ),

                'lot_size' => array(
                    'type' => 'number',
                    'label' => __( 'Lot size', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'search_range_mode' => true,
                    'units' => 'lot_size_unit',
                    'formatter' => 'area',
                    'loop_callback' => array(
                        'callback' => 'es_the_formatted_lot_size',
                        'args' => array( '', ' ', false ),
                    ),
                    'options' => array( 'step' => 0.1 ),
                    'section' => 'es-info',
                    'import_support' => true,
                ),

                'lot_size_unit' => array(
                    'type' => 'list',
                    'values' => $es_settings::get_setting_values( 'unit' ),
                    'template' => true,
                    'label' => false,
                    'skip_search' => true,
                    'default_value' => $es_settings->unit,
                ),

                'year_built' => array(
                    'type' => 'text',
                    'label' => __( 'Year built', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'section' => 'es-info',
                    'import_support' => true,
                ),

                'post_content' => array(
                    'type' => 'wp_editor',
                    'label' => __( 'Description', 'es-plugin' ),
                    'tab' => 'es-basic-info',
                    'system' => true,
                    'import_support' => true,
                ),

                'address' => array(
                    'type' => 'autocomplete',
                    'label' => __( 'Address', 'es-plugin' ),
                    'tab' => 'es-address',
                    'options' => array( 'placeholder' => __( 'Address, City, ZIP', 'es-plugin' ) ),
                    'autocomplete_action' => 'es_address_autocomplete',
                    'import_support' => true,
                ),

                'latitude' => array(
                    'type' => 'number',
                    'label' => __( 'Latitude', 'es-plugin' ),
                    'tab' => 'es-address',
                    'options' => array( 'step' => 'any' ),
                    'import_support' => true,
                ),

                'video' => array(),

                'longitude' => array(
                    'type' => 'number',
                    'tab' => 'es-address',
                    'label' => __( 'Longitude', 'es-plugin' ),
                    'options' => array( 'step' => 'any' ),
                    'import_support' => true,
                ),

                'address_components' => array(
                    'type' => 'hidden',
                    'tab' => 'es-address',
                    'label' => false,
                ),

                'gallery' => array(
                    'label' => __( 'Gallery', 'es-plugin' ),
                    'type' => 'custom',
                    'gallery' => true,
                    'file' => true,
                    'tab' => 'es-media',
                    'template' => ES_PLUGIN_PATH . ES_DS . 'admin' . ES_DS . 'templates' .
                        ES_DS . 'property' . ES_DS . 'media.php',
                    'import_support' => true,
                ),

                'country' => array(
                    'type' => 'list',
                    'values' => array( __( 'Country', 'es-plugin' ) ),
                    'options' => array(
                        'class' => 'js-es-location',
                        'data-type' => Es_Search_Location::LOCATION_COUNTRY_TYPE,
                        'disabled' => 'disabled'
                    ),
                ),

                'state' => array(
                    'type' => 'list',
                    'values' => array( __( 'State', 'es-plugin' ) ),
                    'options' => array(
                        'class' => 'js-es-location',
                        'data-type' => Es_Search_Location::LOCATION_STATE_TYPE,
                        'disabled' => 'disabled'
                    ),
                ),

                'city' => array(
                    'type' => 'list',
                    'values' => array( __( 'City', 'es-plugin' ) ),
                    'options' => array(
                        'class' => 'js-es-location',
                        'data-type' => Es_Search_Location::LOCATION_CITY_TYPE,
                        'disabled' => 'disabled'
                    ),
                ),

                'neighborhood' => array(
                    'type' => 'list',
                    'values' => array( __( 'Neighborhood', 'es-plugin' ) ),
                    'options' => array(
                        'class' => 'js-es-location',
                        'data-type' => Es_Search_Location::LOCATION_NEIGHBORHOOD_TYPE,
                        'disabled' => 'disabled'
                    )
                ),

                'street' => array(
                    'type' => 'list',
                    'values' => array( __( 'Street', 'es-plugin' ) ),
                    'options' => array(
                        'class' => 'js-es-location',
                        'data-type' => Es_Search_Location::LOCATION_STREET_TYPE,
                        'disabled' => 'disabled'
                    ),
                ),
            );

            // Add taxonomies as property fields (for search).
            $taxonomies = Es_Taxonomy::get_taxonomies_list();

            if ( ! empty( $taxonomies ) ) {
                foreach ( $taxonomies as $key => $taxonomy ) {
                    if ( 'es_labels' == $taxonomy  ) continue;

                    $tax = new Es_Taxonomy( $taxonomy );
                    $sys_tax = $tax->get();
                    $terms = (array) get_terms( $sys_tax->name, array( 'hide_empty' => false ) );

                    $values = array();

                    if ( $terms ) {
                        foreach ( $terms as $term ) {
                            $values[ $term->term_id ] = __( $term->name, 'es-plugin' );
                        }
                    }

                    $loop_callback = $taxonomy == 'es_status' ? array( 'callback' => 'es_the_status_list', 'args' => array(
                        '', ' ', '', false
                    ) ) : null;

                    $loop_callback = $taxonomy == 'es_type' ? array( 'callback' => 'es_the_types', 'args' => array(
                        '', ' ', '', false
                    ) ) : $loop_callback;

                    $loop_callback = $taxonomy == 'es_rent_period' ? array( 'callback' => 'es_the_rent_period', 'args' => array(
                        '', ' ', '', false
                    ) ) : $loop_callback;

                    $fields = Es_Object::push_column( array(
                        $taxonomy => array(
                            'label' => __( $tax->get()->label, 'es-plugin' ),
                            'system_type' => 'taxonomy',
                            'type' => 'list',
                            'prompt' => sprintf( __( '-- Choose %s --', 'es-plugin' ), $tax->get()->label ),
                            'system' => true,
                            'section' => in_array( $taxonomy, array( 'es_rent_period', 'es_status', 'es_type' ) ) ? 'es-info' : null,
                            'loop_callback' => $loop_callback,
                            'values' => $values,
                            'import_support' => true,
                            'allow_empty' => 'es_amenities' == $taxonomy || 'es_feature' == $taxonomy ? true : false,
                            'tab' => 'es_amenities' == $taxonomy || 'es_feature' == $taxonomy ? 'es-featured' : 'es-basic-info',
                            'search_operator' => 'IN',
                            'options' => array(
                                'class' => 'es-select2-tags',
                                'multiple' => true,
                                'data-placeholder' => __( $tax->get()->label, 'es-plugin' ),
                            ) ),
                    ), $fields, 9 );

                    if ( $taxonomy == 'es_feature' || $taxonomy == 'es_amenities' ) {
                        $fields[ $taxonomy ]['search_operator'] = 'AND';
                    }
                }
            }

            $labels = self::get_labels_list();

            if ( ! empty( $labels ) && $es_settings->show_labels ) {
                $i = 7;
                foreach ( $labels as $term ) {
                    $fields = Es_Object::push_column( array(
                        $term->slug => array(
                            'type' => 'checkbox',
                            'tab' => 'es-basic-info',
                            'import_support' => true,
                            'label' => __( $term->name, 'es-plugin' ),
                            'options' => array(
                                'value' => 1,
                                'class' => 'es-switch-input'
                            ),
                        ),
                    ), $fields, $i++ );
                }
            }

            $builder_fields = Es_FBuilder_Helper::get_fields( 'property' );

            $builder_fields['custom'] = array(
                'type' => 'custom',
                'import_support' => true,
                'label' => __( 'Custom', 'es-plugin' ),
                'tab' => 'es-basic-info',
                'template' => ES_PLUGIN_PATH . ES_DS . 'admin' . ES_DS . 'templates' .
                    ES_DS . 'property' . ES_DS . 'custom-fields.php',
            );

            $session_storage->set( 'property_fields', $fields );
            $session_storage->set( 'property_builder_fields', $builder_fields );
        }
        return apply_filters( 'es_property_get_fields', array_merge( $fields, $builder_fields ) );
    }

    /**
     * Return list of labels.
     *
     * @return array|int|WP_Error
     */
    public static function get_labels_list()
    {
        return get_terms( array( 'taxonomy' => 'es_labels', 'hide_empty' => false ) );
    }

    /**
     * Return es_labels taxonomy colors.
     *
     * @return mixed
     */
    public static function get_label_colors()
    {
        $standard_labels = es_get_standard_label_names();
        $colors = ! empty ( $standard_labels ) ? array_keys( $standard_labels ) : array();
        $colors[] = '#c40808';
        return apply_filters( 'es_get_label_colors', $colors );
    }

    /**
     * Save property data. Used for save_post hook.
     *
     * @param $post_id
     * @param $post
     *
     * @return void
     */
    public static function save( $post_id, $post )
    {
        if ( $post->post_type == static::get_post_type_name() ) {
            // Initialize property object.
            $property = new static( $post_id );
            // Get property fields data from the post request.
            $data = filter_input( INPUT_POST, 'property', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY );
            // Save these fields.
            $property->save_fields( $data );

            // Saving custom property data fields (that created manually).
            $keys = filter_input(INPUT_POST, 'es_custom_key', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);
            $values = filter_input(INPUT_POST, 'es_custom_value', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

            if ( ! empty( $data['system'] ) ) {
                $property->save_terms( $data['system'] );
            }

            $keys = $keys ? $keys : array();
            $values = $values ? $values : array();

            $custom_data = array_combine( $keys, $values );
            $property->save_custom_data( $custom_data );
        }
    }

    /**
     * @param $data
     */
    public function save_terms( $data )
    {
        $taxonomies = Es_Taxonomy::get_taxonomies_list();
        foreach ( $taxonomies as $taxonomy_name ) {
            if ( ! empty( $data[ $taxonomy_name ] ) ) {
                if ( ! $data[ $taxonomy_name ] ) continue;

                $data[ $taxonomy_name ] = is_string( $data[ $taxonomy_name ] ) ? array( $data[ $taxonomy_name ] ) : $data[ $taxonomy_name ];

                $data[ $taxonomy_name ] = array_map( function( $value ) {
                    return is_numeric( $value ) ? intval( $value ) : $value;
                }, $data[ $taxonomy_name ] );

                wp_set_object_terms( $this->getID(), $data[ $taxonomy_name ], $taxonomy_name );
            }
        }
    }

    /**
     * Return post type name.
     *
     * @return mixed
     */
    public static function get_post_type_name()
    {
        return apply_filters( 'es_property_post_type_name', 'properties' );
    }

    /**
     * Find properties ids using address.
     *
     * @param $address
     * @return array
     */
    public static function find_by_address( $address )
    {
        global $wpdb;

        $post_ids = $wpdb->get_col( "SELECT post_id 
            FROM $wpdb->postmeta 
            WHERE meta_key = 'es_property_address' 
            AND LOWER(`meta_value`) 
            LIKE '%" . strtolower( $address ) . "%'" );

        if ( empty( $post_ids ) ) {
            $components = ES_Address_Components::find_components( $address );

            if ( $components ) {
                $meta_query = array( 'relation' => 'OR' );
                foreach ( $components as $component ) {
                    $meta_query[] = array(
                        'key' => '_address_component_' . $component->type,
                        'value' => $component->id,
                        'compare' => '='
                    );
                }
                $posts = new WP_Query( array(
                    'post_type' => Es_Property::get_post_type_name(),
                    'post_status' => 'publish',
                    'meta_query' => $meta_query,
                    'fields' => 'ids',
                ) );

                $post_ids = $posts->get_posts();
            }
        }

        return $post_ids;
    }

    /**
     * @return string
     */
    public function get_base_field_name()
    {
        return 'property';
    }

    /**
     * Delete post meta data.
     *
     * @param $post_id
     */
    public static function delete( $post_id ) {
        $post = get_post( $post_id );

        if ( $post instanceof WP_Post && $post->post_type == self::get_post_type_name() ) {
            $property = es_get_property( $post->ID );
            $gallery = $property->gallery;

            // Remove attachments.
            if ( $gallery ) {
                array_walk( $gallery, function( $id ) use ( $post_id ) {
                    $check = false;

                    global $wpdb;
                    $results = $wpdb->get_results( "SELECT meta_value FROM {$wpdb->postmeta} 
                        WHERE meta_key='es_property_gallery' 
                        AND post_id NOT IN ($post_id) 
                        AND meta_value IS NOT NULL 
                        AND meta_value LIKE '%" . $id . "%'" );

                    if ( ! empty( $results ) ) {
                        foreach ( $results as $result ) {
                            $meta = ! empty( $result->meta_value ) ? unserialize( $result->meta_value ) : array();

                            if ( is_array( $meta ) && in_array( $id, $meta ) ) {
                                $check = true;
                                break;
                            }
                        }
                    }

                    if ( ! $check ) {
                        wp_delete_attachment( $id, true );
                    }
                } );
            }

            // Remove address components.
            $components = json_decode( $property->address_components );
            if ( $components ) {
                global $wpdb;
                $prefix = '_address_component_';

                foreach ( $components as $component ) {
                    if ( empty( $component->types[0] ) ) continue;

                    $type = $component->types[0];
                    $key = $prefix . $type;
                    $component = ES_Address_Components::get_property_component( $property->getID(), $type );

                    if ( $component ) {
                        $check_meta = $wpdb->get_var( "SELECT COUNT(*) FROM {$wpdb->postmeta} 
                                                      WHERE meta_key = '{$key}' AND meta_value = '{$component->id}'" );

                        if ( $check_meta == 1 ) {
                            ES_Address_Components::delete_component( $component->id );
                        }
                    }
                }
            }
        }
    }

}
