<?php

/**
 * Class Es_Subscription_Settings
 */
class Es_Subscription_Init
{
    /**
     * Start subscription functional functional.
     *
     * @return void
     */
    public static function run()
    {
        $o = new self();
        $o->load_classes();
        $o->filters();
        $o->actions();
        $o->init();
    }

    /**
     * Initialize subscription settings.
     *
     * @return void
     */
    public function init()
    {
        global $es_settings;

        Es_Subscription_List_Page::init();
        Es_Subscription_Metabox::init();
        Es_Subscription_Table::init();
        Es_Order_List_Page::init();

        if ( $es_settings->payment_method ) {
            $class = es_subscription_get_payment_method_class();
            $class->init();
        }
    }

    /**
     * Load subscription classes.
     *
     * @return void.
     */
    public function load_classes()
    {
        $root_classes = ES_PLUGIN_PATH . '/subscriptions/classes/';
        $root_pages = ES_PLUGIN_PATH . '/subscriptions/classes/pages/';
        $root_payments = ES_PLUGIN_PATH . '/subscriptions/classes/payments/';

        $classes = array(
            ES_PLUGIN_PATH . '/subscriptions/functions.php',
            $root_classes . 'class-subscription.php',
            $root_classes . 'class-order.php',
            $root_classes . 'class-subscription-table.php',
            $root_classes . 'class-subscription-metabox.php',
            $root_pages . 'class-subscription-list-page.php',
            $root_pages . 'class-order-list-page.php',
            $root_payments . 'class-subscription-payment-method.php',
            $root_payments . 'class-paypal-subscription-payment-method.php',
        );

        foreach ( $classes as $class_path ) {
            include ( $class_path );
        }
    }

    /**
     * Add actions for subscription settings.
     *
     * @return void
     */
    public static function actions()
    {
        if ( es_subscription_is_enabled() ) {
            add_action( 'init', array( __CLASS__, 'register_post_types' ) );
            add_action( 'wp_dashboard_setup', array( __CLASS__, 'add_dashboard_widget' ) );
            add_action( 'wp_enqueue_scripts', array( __CLASS__, 'enqueue_styles' ) );
            add_action( 'wp_enqueue_scripts', array( __CLASS__, 'enqueue_scripts' ) );
            add_action( 'transition_post_status', array( __CLASS__, 'property_save_check_published' ), 11, 3 );
            add_action( 'before_delete_post', array( __CLASS__, 'property_deleted' ), 11, 1 );
            add_action( 'delete_post', array( __CLASS__, 'property_deleted' ), 10, 1 );
			add_action( 'es_subscription_expired', array( __CLASS__, 'send_expired_email' ), 10, 1 );

            if ( is_admin() ) {
                add_action( 'admin_notices', array( __CLASS__, 'admin_notices' ) );
            }

            if ( es_subscription_is_enabled() ) {
                add_action( 'init', array( __CLASS__, 'check_user_subscription' ) );
                add_action( 'es_check_agent_subscriptions', array( __CLASS__, 'check_agent_subscriptions' ) );
            }
        }

        add_action( 'es_settings_save', array( __CLASS__, 'enable_subscription_action' ), 10, 3 );
    }

    /**
     * Send expired subscription email for user.
     *
     * @param $user_id
     */
    public static function send_expired_email( $user_id = null )
    {
        global $es_settings;

        if ( $es_settings->expired_email_body && $es_settings->expired_email_subject ) {
            $user = es_get_agent( $user_id )->get_entity();
            $mail = wp_mail( $user->user_email, $es_settings->expired_email_subject, $es_settings->expired_email_body );
        }
    }

    /**
     * Add filters for subscription settings.
     *
     * @return void
     */
    public function filters()
    {
        add_filter( 'es_get_available_settings', array( $this, 'available_settings' ) );
        add_filter( 'es_settings_get_tabs', array( $this, 'settings_tab' ) );
        if ( es_subscription_is_enabled() ) {
            add_filter( 'wp_die_handler', array( __CLASS__, 'die_handler' ), 10, 1 );
            add_filter( 'wp_die_handler', array( __CLASS__, 'die_handler' ), 10, 1 );
            add_filter( 'es_render_field', array( __CLASS__,  'featured_field_render' ), 10, 5 );
            add_action( 'es_before_save_property_data', array( __CLASS__, 'property_save_check_featured' ), 10, 2 );
        }
    }

    /**
     * Manage featured field for subscriptions
     *
     * @param $content
     * @param $id
     * @param $field
     *
     * @return string
     */
    public static function featured_field_render( $content, $id, $field, $field_name, $options )
    {
        // Check if subscription is enabled and check for featured field.
        if ( es_subscription_is_enabled() && $id == 'featured' && es_subscription_is_user_has_subscription() ) {

            // Get current post.
            global $post;
            $property = es_get_property( $post->ID );

            // Get current agent.
            $agent = es_get_agent();

            // Num of published featured posts.
            $featured_published = $agent->subscription_featured_posts;

            // Limit for featured posts.
            $featured_limit = $agent->subscription_featured_qty;

            if ( $featured_limit <= $featured_published && ! $property->featured ) {
                $content = '<p class="property-data-field es-field-type-' . $field['type'] . '"><label><span class="es-settings-label">' . $field['label'] . '</span>';
                $content .= '<input disabled type="' . $field['type'] . '" name="' . $field_name . '[' . $id . ']" id="es-' . $id .'-input"' . $options . '/>';
                $content .= '</label></p>';
            }
        }

        return $content;
    }

    /**
     * @param null $user_id
     */
    public static function check_user_subscription( $user_id = null )
    {
        $agent = es_get_agent( $user_id );
        $entity = $agent->get_entity();

        if ( ! empty( $entity ) ) {
            if ( es_subscription_is_user_subscription_expired( $agent->getID() ) || (
                ! es_subscription_is_user_has_subscription( $agent->getID() ) && current_user_can( 'publish_es_properties' ) )
                ) {
                es_subscription_unactivate_user_subscription( $agent->getID() );
                es_subscription_change_user_caps( $user_id, ES_REMOVE_CAPS );
            }
        }
    }

	/**
	 * Check by schedule each hour is system have users with expired subscriptions and disable it.
     *
     * @return void
	 */
	public static function check_agent_subscriptions()
	{
		if ( es_subscription_is_enabled() ) {
            $q = new WP_User_Query( array(
                'role' => Es_Agent::get_role_name(),
                'meta_key' => 'es_agent_subscription_expired',
                'meta_compare' => 'EXISTS',
            ) );

			$users = $q->get_results();

			if ( $users ) {
				foreach ( $users as $user ) {
					static::check_user_subscription( $user->ID );
				}
			}
		}
	}

    /**
     * Disable or enable subscription for agents.
     *
     * @param $key
     * @param $value
     * @param $prev_value
     */
    public static function enable_subscription_action( $key, $value, $prev_value )
    {
        if ( $value == $prev_value || 'enabled_subscription' != $key ) return;

        // Subscription state.
        $enabled = (bool) $value;

        // Query for users.
        $args = array( 'role' => Es_Agent::get_role_name() );
        $users = get_users( $args );

        global $es_settings;

        if ( $users ) {
            $cap_publish = apply_filters( 'es_automatic_publishing_permission_name', 'publish_es_properties' );

            foreach ( $users as $user ) {
                es_subscription_update_user_properties_status( $user->ID );

                // If subscription is enabled.
                if ( $enabled ) {
                    if ( es_subscription_is_user_subscription_expired( $user->ID ) ) {
                        es_subscription_unsubscribe_user( $user->ID );
                    } else if ( es_subscription_is_user_has_subscription( $user->ID ) ) {
                        $agent = es_get_agent( $user->ID );

                        es_subscription_change_user_caps( $user->ID );
                        $agent->save_field_value( 'subscription_published_posts', 0 );
                        $agent->save_field_value( 'subscription_featured_posts', 0 );
                    }

                // if subscription is disabled.
                } else {
                    es_subscription_change_user_caps( $user->ID, ES_RESTORE_CAPS );

                    if ( $es_settings->listings_publishing == 'manual' ) {
                        es_subscription_change_user_caps( $user->ID, ES_REMOVE_CAPS, array( $cap_publish ) );
                    }
                }
            }
        }

        if ( $enabled ) {
            // Create schedule for activate / deactivate agent subscriptions.
            if ( ! wp_next_scheduled ( 'es_check_agent_subscriptions' )) {
                wp_schedule_event(time(), 'hourly', 'es_check_agent_subscriptions');
            }

            es_subscription_change_role_caps( ES_REMOVE_CAPS );

            if ( function_exists( 'es_subscription_create_paypal_response_page' ) ) {
                es_subscription_create_paypal_response_page();
            }

            if ( function_exists( 'es_subscription_create_subscription_table_page' ) ) {
                es_subscription_create_subscription_table_page();
            }
        } else {
            $caps = null;

            if ( $es_settings->listings_publishing == 'manual' ) {
                $caps = array( 'create_es_properties' );
            }

            es_subscription_change_role_caps( ES_RESTORE_CAPS, Es_Agent::get_role_name(), $caps );
            // Remove schedule for activate / deactivate agent subscriptions.
            wp_clear_scheduled_hook( 'es_check_agent_subscriptions' );
        }
    }

    /**
     * @return void
     */
    public static function enqueue_scripts()
    {
        // Base front script.
        wp_register_script(
            'es-subscription-table-script',
            ES_PLUGIN_URL . 'subscriptions/assets/js/custom/es-subscription-table.js',
            array ( 'jquery' )
        );
        wp_enqueue_script( 'es-subscription-table-script' );
    }

    /**
     * Enqueue styles.
     *
     * @return void
     */
    public static function enqueue_styles()
    {
        // Register base styles for the plugin.
        wp_register_style( 'es-subscription-table-style', ES_PLUGIN_URL . 'subscriptions/assets/css/custom/table.css' );
        wp_enqueue_style( 'es-subscription-table-style' );
    }

    /**
     * Display admin notices for subscriptions feature.
     *
     * @return void
     */
    public static function admin_notices()
    {
        global $post, $wp_query;
        $notice_type = 'notice-warning';

        if ( es_subscription_is_enabled() && $wp_query && $post && ! $wp_query->post_count && $post->post_type == Es_Property::get_post_type_name() ) {
            $agent = es_get_agent( $post->post_author );
            if ( ! $agent->get_entity()->has_cap( 'publish_es_properties' ) ) : ?>
                <div class="notice <?php echo $notice_type; ?> is-dismissible">
                    <p>
                        <b><?php _e( 'Estatik', 'es-plugin' ); ?></b>:
                        <?php echo __( 'Agent of this post has no permission for publish this property.', 'es-plugin' ); ?>
                    </p>
                </div>
            <?php endif;
        }
    }

    /**
     * Manage post saving for users.
     *
     * @param $new_status
     * @param null $old_status
     * @param null|WP_Post $post
     */
    public static function property_save_check_published( $new_status, $old_status = null, $post = null )
    {
        if ( $new_status == $old_status || Es_Property::get_post_type_name() != $post->post_type ) return;

        $agent = es_get_agent( $post->post_author );
        // Num of published posts.
        $already_published = $agent->subscription_published_posts;
        $limit_published = $agent->subscription_property_qty;

        // If post is published.
        if ( $new_status != $old_status && $new_status == 'publish' ) {

            if ( $already_published < $limit_published ) {
                $agent->save_field_value( 'subscription_published_posts', ++$already_published );

                if ( $already_published == $limit_published ) {
                    es_subscription_change_user_caps( $agent->getID(), ES_REMOVE_CAPS );
                }
            } else {
                global $wpdb;

                if ( current_user_can( 'edit_others_es_properties' ) ) {
                    $agent->save_field_value( 'subscription_published_posts', ++$already_published );
                } else {
                    $wpdb->update( $wpdb->posts, array( 'post_status' => 'draft' ), array( 'ID' => $post->ID ) );

                    $messenger = new Es_Messenger( 'manage-property' );
                    $messenger->set_message( __( "You subscription plan doesn't allow to publish more listings.", "es-plugin" ), 'warning' );
                }
            }
        }

        // If post is unpublished.
        if ( $new_status != $old_status && $old_status == 'publish' ) {
            self::property_deleted( $post->ID, true );
        }
    }

    /**
     * Check for featured posts.
     *
     * @param array $post_data
     *    Property meta data array.
     * @param WP_Post $post
     *
     * @return array
     */
    public static function property_save_check_featured( $post_data, $post )
    {
        if ( Es_Property::get_post_type_name() == $post->post_type ) {
            $agent = es_get_agent();
            $property = es_get_property( $post->ID );

            $is_featured_before = $property->featured;
            $is_featured_after = ! empty( $post_data['featured'] ) ? $post_data['featured'] : 0;

            $featured = $agent->subscription_featured_posts;
            $featured_limit = $agent->subscription_featured_qty;

            // If prev featured value is not equal for value before.
            if ( $is_featured_after != $is_featured_before ) {

                if ( $is_featured_before ) {
                    $agent->save_field_value( 'subscription_featured_posts', $featured > 0 ? --$featured : 0 );
                } else if ( $is_featured_after ) {
                    if ( $featured >= $featured_limit ) {
                        if ( ! current_user_can( 'edit_others_es_properties' ) ) {
                            $post_data['featured'] = '';
                            $messenger = new Es_Messenger( 'manage-property' );
                            $messenger->set_message( __( "You subscription plan doesn't allow to publish more featured listings.", "es-plugin" ), 'warning' );
                        }
                    } else {
                        $agent->save_field_value( 'subscription_featured_posts', ++$featured );
                    }
                }
            }
        }

        return $post_data;
    }

    /**
     * @param $post_id
     * @param bool $force
     */
    public static function property_deleted( $post_id, $force = false )
    {
        $post = get_post( $post_id );

        if ( $post->post_type == Es_Property::get_post_type_name() && ( $post->post_status == 'publish' || $force ) ) {
            $agent = es_get_agent( $post->post_author );
            $already_published = $agent->subscription_published_posts;
            $limit_published = $agent->subscription_property_qty;
            $property = es_get_property( $post_id );

            $already_published = $already_published ? --$already_published : $already_published;
            $agent->save_field_value( 'subscription_published_posts', $already_published );

            if ( $property->featured ) {
                $agent->save_field_value(
                    'subscription_featured_posts',
                    $agent->subscription_featured_posts ? --$agent->subscription_featured_posts : 0
                );
            }

            if ( $limit_published > $already_published ) {
                es_subscription_change_user_caps( $agent->getID(), ES_RESTORE_CAPS );
            }
        }
    }

    /**
     * Add subscription dashboard widget.
     *
     * @return void
     */
    public static function add_dashboard_widget()
    {
        $agent = new Es_Agent( get_current_user_id() );

        if ( $agent->es_subscription_active ) {
            wp_add_dashboard_widget(
                'es_subscription_widget',
                __( 'Estatik Subscription', 'es-plugin' ),
                array( __CLASS__, 'dashboard_widget_display' )
            );
        }
    }

    /**
     * Render subscription dashboard widget.
     *
     * @return void
     */
    public static function dashboard_widget_display()
    {
        $agent = new Es_Agent( get_current_user_id() );

        if ( ! empty( $agent->es_subscription_order_id ) ) {
            $order = es_get_order( $agent->es_subscription_order_id );

            $subscription = $order->get_subscription();
            $temp = $order->subscription_temp;

            $path = apply_filters( 'es_dashboard_subscription_widget_path', ES_PLUGIN_PATH . '/subscriptions/templates/widgets/dashboard-subscription.php' );

            include( $path );
        }
    }

    public static function die_handler( $callback )
    {
        return $callback;
    }

    /**expired_email_subject
     * Add new settings for subscription feature.
     *
     * @param $settings
     * @return mixed
     */
    public function available_settings($settings)
    {
        $settings['enabled_subscription'] = array( 'default_value' => 0 );
        $settings['payment_method'] = array( 'default_value' => 'paypal_subscription' );
        $settings['expired_email_body'] = array( 'default_value' => 'Hello, Your subscription plan period ends today. Please renew your subscription
                                                                     and click <a> here &gt;&gt;</a> to proceed. Otherwise your listings will be unpublished.
                                                                     Thank you.' );
        $settings['expired_email_subject'] = array( 'default_value' => __( 'Subscription is expired.', 'es-plugin' ) );
        $settings['subscription_checkout_page_id'] = array( 'default_value' => 0 );

        return $settings;
    }

    /**
     * Add new subscription settings tab to the settings page.
     *
     * @param $tabs
     * @return mixed
     */
    public function settings_tab( $tabs )
    {
        $tabs['subscription'] = array(
            'label' => __( 'Subscriptions', 'es-plugin' ),
            'template' => ES_PLUGIN_PATH . '/subscriptions/templates/settings/tab.php'
        );

        return $tabs;
    }

    /**
     * Register subscription post types.
     *
     * @return void
     */
    public static function register_post_types()
    {
        register_post_type( 'es_subscription', array(
            'label' => __( 'Subscription', 'es-plugin' ),
            'labels' => array(
                'name' => __( 'Subscriptions', 'es-plugin' ),
                'edit_item' => __( 'Edit subscription', 'es-plugin' ),
            ),
            'public' => true,
            'show_in_menu' => false,
            'has_archive' => false,
            'publicly_queryable'  => false,
            'supports' => array( 'title', 'excerpt' ),
            'rewrite' => array(
                'slug' => 'subscription',
            ),
        ) );

        register_post_type( 'es_order', array(
            'label' => __( 'Order', 'es-plugin' ),
            'labels' => array(
                'name' => __( 'Orders', 'es-plugin' ),
            ),
            'public' => true,
            'show_in_menu' => false,
            'has_archive' => false,
            'supports' => array( 'title', 'excerpt' ),
            'publicly_queryable'  => false,
            'rewrite' => array(
                'slug' => 'order',
            ),
        ) );

        flush_rewrite_rules();
    }
}
