<?php

/**
 * Class Es_Property_List_Page
 *
 * Customize properties post type admin list page.
 */
class Es_Order_List_Page extends Es_Object
{
    /**
     * Custom property post type slug.
     *
     * @var string
     */
    protected $_post_type = 'es_order';

    /**
     * Initialize actions.
     *
     * @return void
     */
    public function actions()
    {
        // Add action for render custom columns values.
        add_action( 'manage_' . $this->_post_type . '_posts_custom_column' , array( $this, 'add_columns_values' ), 10, 2 );
        // Enqueue styles for our page.
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_styles' ) );
    }

    /**
     * Initialize filters.
     *
     * @return void
     */
    public function filters()
    {
        // Add custom columns to the list table.
        add_filter( 'manage_' . $this->_post_type . '_posts_columns', array( $this, 'add_columns' ) );
        // Remove standard filters from our content type.
        add_filter( 'views_edit-' . $this->_post_type, array( $this, 'remove_views' ) );
        // Remove bulk actions from our content type.
        add_filter( 'bulk_actions-edit-' . $this->_post_type, array( $this, 'remove_views' ) );
        // Global js variables.
        add_filter( 'es_global_js_variables', array( $this, 'add_js_variables' ), 10, 1 );
        // Customize manage post buttons.
        add_filter( 'post_row_actions', array( $this, 'post_row_actions' ), 10, 2 );

        if ( is_admin() ) {
            add_filter( 'the_title', array( $this, 'customize_title' ), 10, 2 );
        }
    }

    public function customize_title( $title, $post ) {
        $post = get_post( $post );
        if ( $post->post_type == $this->_post_type ) {
            $order = es_get_order( $post->ID );
            if ( $subscription = $order->get_subscription() ) {
                $title = $subscription->price ? $title . ' | ' . __( 'Price', 'es-plugin' ) . ': ' .  $subscription->price . ' ' . $subscription->currency : $title;
            }
        }
        return $title;
    }

    /**
     * @param $actions
     * @param $post
     * @return array
     */
    public function post_row_actions( $actions, $post ) {
        if ( $post->post_type == $this->_post_type ) {
            $actions = array();
        }
        return $actions;
    }

    /**
     * Add JS variables for this page.
     *
     * @param $data
     * @return mixed
     */
    public function add_js_variables( $data ) {
        if ( current_user_can( 'activate_plugins' ) ) {
            $data['html']['logo'] = es_get_logo();
        }

        return $data;
    }

    /**
     * Add custom columns for property post type.
     *
     * @param $columns
     *    Array of name => label columns.
     * @return mixed|array
     *    Return customized columns array.
     */
    public function add_columns( $columns )
    {
        $columns = array();

        $columns['post_id'] = false;
        $columns['title'] = __( 'Name', 'es-plugin' );
        $columns['order_status'] = __( 'Order status', 'es-plugin' );
        $columns['subscription'] = __( 'Subscription', 'es-plugin' );
        $columns['price'] = __( 'Price', 'es-plugin' );
        $columns['agent'] = __( 'Agent', 'es-plugin' );

        return $columns;
    }

    /**
     * Render value for specific column.
     *
     * @param $column
     *    Column name.
     * @param $post_id
     *    WP_Post ID.
     *
     * @return void
     */
    public function add_columns_values( $column, $post_id )
    {
        $order = es_get_order( $post_id );
        $subscription = $order->get_subscription();
        $agent = $order->get_agent();

        if ( 'post_id' == $column )
            echo $post_id;

        if ( 'order_status' == $column ) {
            if ( $order->status ) {
                echo $order->status;
            } else {
                echo '<span style="color: #999999">(' . __( 'Not set', 'es-plugin' ) . ')</span>';
            }
        }

        if ( 'subscription' == $column ) {
            if ( $subscription ) {
                echo $subscription->get_entity()->post_title;
            } else {
                echo '<span style="color: #999999">(' . __( 'Not set', 'es-plugin' ) . ')</span>';
            }
        }

        if ( 'price' == $column ) {
            if ( $subscription ) {
                echo $subscription->price . ' ' . $subscription->currency;
            } else {
                echo '<span style="color: #999999">(' . __( 'Not set', 'es-plugin' ) . ')</span>';
            }
        }

        if ( 'agent' == $column ) {
            if ( $agent ) {
                echo $agent->get_entity()->user_email;
            } else {
                echo '<span style="color: #999999">(' . __( 'Not set', 'es-plugin' ) . ')</span>';
            }
        }
    }

    /**
     * Register and enqueue styles for our page.
     *
     * @return void
     */
    public function enqueue_styles()
    {
        wp_register_style( 'es-admin-order-list-style', ES_PLUGIN_URL . '/subscriptions/assets/css/custom/order-list.css' );
        wp_enqueue_style( 'es-admin-order-list-style' );
    }

    /**
     * Remove standard filters from our custom post type list page.
     *
     * @return array
     *    Empty array.
     */
    public function remove_views()
    {
        return array();
    }
}
