<?php

/**
 * Class Es_Property_List_Page
 *
 * Customize properties post type admin list page.
 */
class Es_Subscription_List_Page extends Es_Object
{
    /**
     * Custom property post type slug.
     *
     * @var string
     */
    protected $_post_type = 'es_subscription';

    /**
     * Initialize actions.
     *
     * @return void
     */
    public function actions()
    {
        // Add action for render custom columns values.
        add_action( 'manage_' . $this->_post_type . '_posts_custom_column' , array( $this, 'add_columns_values' ), 10, 2 );
        // Enqueue styles for our page.
        add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_styles' ) );
    }

    /**
     * Initialize filters.
     *
     * @return void
     */
    public function filters()
    {
        // Add custom columns to the list table.
        add_filter( 'manage_' . $this->_post_type . '_posts_columns', array( $this, 'add_columns' ) );
        // Remove standard filters from our content type.
        add_filter( 'views_edit-' . $this->_post_type, array( $this, 'remove_views' ) );
        // Remove bulk actions from our content type.
        add_filter( 'bulk_actions-edit-' . $this->_post_type, array( $this, 'remove_views' ) );
        // Global js variables.
        add_filter( 'es_global_js_variables', array( $this, 'add_js_variables' ), 10, 1 );
        // Customize manage post buttons.
        add_filter( 'post_row_actions', array( $this, 'post_row_actions' ), 10, 2 );
    }

    /**
     * Add JS variables for this page.
     *
     * @param $data
     * @return mixed
     */
    public function add_js_variables( $data ) {
        if ( current_user_can( 'activate_plugins' ) ) {
            $data['html']['logo'] = es_get_logo();
        }

        return $data;
    }

    /**
     * @param $actions
     * @param $post
     * @return array
     */
    public function post_row_actions( $actions, $post ) {
        if ( $post->post_type == $this->_post_type ) {
            // Customize edit link.
            ob_start();
            edit_post_link( '<i class="fa fa-pencil" aria-hidden="true"></i>' );
            $actions['edit'] = ob_get_clean();

            // Customize remove link.
            $actions['trash'] = '<a href="' . get_delete_post_link( $post->ID ) . '">
                <i class="fa fa-trash" aria-hidden="true"></i></a>';

            unset( $actions['inline hide-if-no-js'], $actions['view'] );
        }
        return $actions;
    }

    /**
     * Add custom columns for property post type.
     *
     * @param $columns
     *    Array of name => label columns.
     * @return mixed|array
     *    Return customized columns array.
     */
    public function add_columns( $columns )
    {
        // Unset left columns.
        unset($columns['author'], $columns['date'], $columns['cb']);

        $columns['title'] = __( 'Name', 'es-plugin' );
        // Add post ID column.
        $columns = static::push_column( array( 'post_id' => false ),  $columns, 0 );
        // Add date column on new position with new label.
        $columns = static::push_column( array( 'property_qty'      => __( 'Properties q-ty', 'es-plugin' ) ), $columns, 4 );
        // Add address column.
        $columns = static::push_column( array( 'featured'   => __( 'Featured', 'es-plugin' ) ),    $columns, 5 );
        // Add category column.
        $columns = static::push_column( array( 'price'  => __( 'Price', 'es-plugin' ) ),   $columns, 6 );

        return $columns;
    }

    /**
     * Render value for specific column.
     *
     * @param $column
     *    Column name.
     * @param $post_id
     *    WP_Post ID.
     *
     * @return void
     */
    public function add_columns_values( $column, $post_id )
    {
        $subscription = es_get_subscription( $post_id );

        if ( 'post_id' == $column )
            echo $post_id;

        if ( 'property_qty' == $column ) {
            echo $subscription->property_qty;
        }

        if ( 'featured' == $column ) {
            echo $subscription->featured_qty;
        }

        if ( 'price' == $column ) {
            echo $subscription->price . ' ' . $subscription->currency;
        }
    }

    /**
     * Register and enqueue styles for our page.
     *
     * @return void
     */
    public function enqueue_styles()
    {
        wp_register_style( 'es-admin-subscription-list-style', ES_PLUGIN_URL . '/subscriptions/assets/css/custom/subscription-list.css' );
        wp_enqueue_style( 'es-admin-subscription-list-style' );
    }

    /**
     * Remove standard filters from our custom post type list page.
     *
     * @return array
     *    Empty array.
     */
    public function remove_views()
    {
        return array();
    }
}
