<?php

require_once( ES_PLUGIN_PATH . '/libraries/paypal-digital-goods/paypal-digital-goods.class.php' );
require_once( ES_PLUGIN_PATH . '/libraries/paypal-digital-goods/paypal-subscription.class.php' );
require_once( ES_PLUGIN_PATH . '/libraries/paypal-digital-goods/paypal-purchase.class.php' );

/**
 * Class Es_PayPal_Subscription_Payment_Method
 */
class Es_PayPal_Subscription_Payment_Method extends Es_Subscription_Payment_Method
{
    const PAYPAL_ACTION_PAID = 'paid';
    const PAYPAL_ACTION_CANCELED = 'canceled';
    const PAYPAL_ACTION_NOTIFY = 'notify';

    /**
     * Initialize payment method.
     *
     * @return void
     */
    public function init()
    {
        $this->filters();
        $this->actions();
    }

    /**
     * Add filters for payment type.
     *
     * @return void
     */
    public function filters()
    {
        add_filter( 'es_get_available_settings', array( $this, 'add_settings' ) );
    }

    /**
     * Add actions for payment type.
     *
     * @return void
     */
    public function actions()
    {
        add_action( 'es_settings_after_subscription_tab', array( $this, 'settings' ) );
        add_action( 'init', array( $this, 'listener_handler' ) );
        add_shortcode( 'es_subscription_paypal_response_page', array( $this, 'shortcode' ) );
    }

    /**
     * [es_subscription_paypal_response_page] shortcode handler.
     *
     * @return void
     */
    public function shortcode()
    {
        $logger = new Es_Messenger( 'es_paypal_messages' );

        if ( $logger->get_messages() ) {
            $logger->render_messages();
        } else {
            echo '<span class="es-empty">' . __( 'Nothing to display.', 'es-plugin' ) . '</span>';
        }

        do_action( 'es_shortcode_after' );
    }

    /**
     * Return paypal RETURN URL.
     *
     * @return mixed
     */
    public function get_return_url()
    {
        return apply_filters( 'es_get_paypal_return_url', add_query_arg( 'paypal', self::PAYPAL_ACTION_PAID, get_site_url() . '/' ) );
    }

    /**
     * Return paypal CANCEL URL.
     *
     * @return mixed|string
     */
    public function get_cancel_url()
    {
        return apply_filters( 'es_get_paypal_cancel_url', add_query_arg( 'paypal', self::PAYPAL_ACTION_CANCELED, get_site_url() . '/' ) );
    }

    /**
     * Check PayPal IPN validity.
     *
     * @return bool
     */
    public function validate_ipn() {
        // Get received values from post data
        $validate_ipn = array( 'cmd' => '_notify-validate' );
        $validate_ipn += wp_unslash( $_POST );

        $params = array(
            'body'        => $validate_ipn,
            'timeout'     => 60,
            'httpversion' => '1.1',
            'compress'    => false,
            'decompress'  => false,
            'user-agent'  => 'Estatik/' . Estatik::getVersion()
        );

        // Post back to get a response.
        $response = wp_safe_remote_post( PayPal_Digital_Goods_Configuration::environment() == 'sandbox' ? 'https://www.sandbox.paypal.com/cgi-bin/webscr' : 'https://www.paypal.com/cgi-bin/webscr', $params );

        // Check to see if the request was valid.
        if ( ! is_wp_error( $response ) && $response['response']['code'] >= 200 && $response['response']['code'] < 300 && strstr( $response['body'], 'VERIFIED' ) ) {
            update_option( 'ipnvalid', 1 );
            return true;
        }

        if ( is_wp_error( $response ) ) {
            update_option( 'es_response_ipn_error',  $response->get_error_message() );
            return false;
        }

        update_option( 'ipnvalid', 0 );

        return false;
    }

    /**
     * Return paypal NOTIFY URL.
     *
     * @return mixed|string
     */
    public function get_notify_url()
    {
        return apply_filters( 'es_get_paypal_notify_url', add_query_arg( 'paypal', self::PAYPAL_ACTION_NOTIFY, get_site_url() . '/' ) );
    }

    /**
     * Return  paypal BUSINESS NAME.
     *
     * @return mixed|string
     */
    public function get_business_name()
    {
        $blog_name = get_bloginfo( 'name' );
        return apply_filters( 'es_get_paypal_business_name', ! empty( $blog_name ) ? $blog_name : __( 'Estatik', 'es-plugin' ) );
    }

    /**
     * Set credentials for payment method.
     *
     * @param null $currency
     */
    public function set_credentials( $currency = null )
    {
        global $es_settings;

        $currency = $currency ? $currency : $es_settings->currency;

        PayPal_Digital_Goods_Configuration::username( $es_settings->payment_username );
        PayPal_Digital_Goods_Configuration::password( $es_settings->payment_password );
        PayPal_Digital_Goods_Configuration::signature( $es_settings->payment_signature );

        PayPal_Digital_Goods_Configuration::return_url( $this->get_return_url() );
        PayPal_Digital_Goods_Configuration::cancel_url( $this->get_cancel_url() );
        PayPal_Digital_Goods_Configuration::notify_url( $this->get_notify_url() );

        PayPal_Digital_Goods_Configuration::environment( $es_settings->payment_mode );
        PayPal_Digital_Goods_Configuration::incontext_url( 'no' );

        PayPal_Digital_Goods_Configuration::business_name( $this->get_business_name() );

        PayPal_Digital_Goods_Configuration::currency( $currency );
    }

    /**
     * Create PayPal product data.
     *
     * @param Es_Subscription $subscription
     *
     * @return array
     */
    protected function create_product_data( Es_Subscription $subscription )
    {
        $post = $subscription->get_entity();

        PayPal_Digital_Goods_Configuration::currency( $subscription->currency );

        $product = array(
            'amount'             => $subscription->price,
            'period'             => $subscription->period,
            'frequency'          => 1,
        );

        if ( ! empty( $subscription->price_renewal ) ) {
            $product['initial_amount'] = $subscription->price_renewal;
        }

        if ( $subscription->trial_period ) {
            $product['trial_period'] = $subscription->trial_period;
        }

        if ( ! empty( $post->post_excerpt ) ) {
            $product['description'] = $post->post_excerpt;
        } else {
            $description[] = 'Plan: ' . $post->post_title;
            foreach ( $product as $key => $value ) {
                $description[] = Es_Html_Helper::generate_label( $key ) . ': ' . $value;
            }
            $product['description'] = implode( ', ', $description );
        }

        return apply_filters( 'es_create_product_data', $product );
    }

    /**
     * Return purshase product.
     *
     * @param Es_Subscription|array $entity
     * @return PayPal_Subscription
     */
    public function create_product( $entity )
    {
        if ( is_array( $entity ) ) {
            $order = new Es_Order();
            $order = $order->get_by_field( 'token', $_GET['token'] );
            $subscription = $order->get_subscription();
            $this->set_credentials( $subscription->currency );
            $product = $entity;
        } else {
            $this->set_credentials( $entity->currency );
            $product = $this->create_product_data( $entity );
        }

        return new PayPal_Subscription( $product );
    }

    /**
     * Submit checkout page handler.
     *
     * @return void
     */
    public function proceed()
    {
        $id = $_POST['es_subscription']['id'];
        $post = get_post( $id );

        try {
            if ($post->post_type == 'es_subscription') {
                $subscription = es_get_subscription( $id );

                $product = $this->create_product( $subscription );
                $temp = $this->create_product_data( $subscription );
                $token = $product->token();

                $order = new Es_Order();

                $order->create(
                    $subscription,
                    wp_get_current_user(),
                    $temp,
                    $token
                );

                wp_redirect( $product->get_checkout_url() );
                exit;
            }
        } catch ( Exception $e ) {
            $message = current_user_can( 'manage_posts' ) ? $e->getMessage() : __( 'PayPal error. Please support with admin.', 'es-plugin' );
            wp_die( $message );
        }
    }

    /**
     * Return setting fields for this payment type.
     *
     * @return mixed
     */
    public function add_settings( $settings )
    {
        $paypal_pages = es_subscription_get_pages();
        $list[] = __( '-- Select PayPal Success Page --', 'es-plugin' );
        if ( ! empty( $paypal_pages ) ) {
            foreach ( $paypal_pages as $page ) {
                $list[ $page->ID ] = $page->post_title;
            }
        }

        return array_merge( $settings, apply_filters( 'es_subscription_payment_method_settings', array(
            'payment_mode' => array( 'default_value' => 'sandbox', 'values' => array(
                'live' => __( 'Live', 'es-plugin' ),
                'sandbox' => __( 'Sandbox (test mode)', 'es-plugin' ),
            ) ),
            'payment_username' => array( 'default_value' => '' ),
            'payment_password' => array( 'default_value' => '' ),
            'payment_signature' => array( 'default_value' => '' ),
            'payment_response_page' => array( 'default_value' => '', 'values' => ! empty( $list ) ? $list : array() ),
        ), $this ) );
    }

    /**
     * Render settings fields for payment method.
     *
     * @return void
     */
    public function settings()
    {
        $template = apply_filters(
            'es_subscription_payment_method_template', ES_PLUGIN_PATH . '/subscriptions/templates/settings/payment.php'
        );

        include $template;
    }

    /**
     * Waiting for PayPal response.
     *
     * @return void
     */
    public function listener_handler()
    {
        // Get PayPal action.
        $paypal_action = filter_input( INPUT_GET, 'paypal' );
        // Get PayPal token.
        $paypal_token = filter_input( INPUT_GET, 'token' );

        $logger = new Es_Messenger( 'es_paypal_messages' );

        $redirect_page = es_subscription_get_current_paypal_response_page();
        $redirect_page = ! empty( $redirect_page ) ? get_permalink( $redirect_page ) : null;

        switch ( $paypal_action ) {

            /**
             * PayPal return URL handler. Start subscription for agent.
             * @throw Exception
             */
            case self::PAYPAL_ACTION_PAID:
                $order = new Es_Order();

                if ( ! empty( $paypal_token ) && ( $order = $order->get_by_field( 'token', $paypal_token ) ) ) {
                    try {
                        $this->set_credentials();

                        $subscription_object = $order->get_subscription();
                        PayPal_Digital_Goods_Configuration::currency( $subscription_object->currenry );

                        // Get temp subscription from order.
                        $subscription = $order->subscription_temp;
                        // Create PayPal product from temp subscription.
                        $product = $this->create_product( $subscription );

                        // Start subscription and get Response from PayPal.
                        $response = $product->start_subscription();

                        if ($response && $response['ACK'] == 'Success') {
                            // Save PayPal profile ID.
                            $order->save_field_value( 'paypal_profile_id', $response['PROFILEID'] );

                            if ( $response['PROFILESTATUS'] == 'ActiveProfile' ) {
                                // Change order status to completed.
                                $order->change_status( 'Active' );

                                es_subscription_subscribe_user( $subscription_object->getID(), $order->get_agent()->getID() );

                                $logger->set_message( sprintf( __( 'Payment successful. Your subscription plan is <b>%s</b>.', 'es-plugin' ), $order->get_subscription()->get_entity()->post_title ), 'success' );
                            } else {
                                $logger->set_message( __( 'Payment successful. Please, wait for PayPal approve.', 'es-plugin' ), 'success' );
                            }
                        } else {
                            $logger->set_message( __( 'Invalid PayPal subscription response.', 'es-plugin' ), 'error' );
                        }
                    } catch ( Exception $e ) {
                        $logger->set_message( $e->getMessage(), 'error' );
                    }
                } else {
                    $logger->set_message( __( 'Invalid PayPal Token or Order.', 'es-plugin' ), 'error' );
                }

                wp_redirect( $redirect_page ); exit;
                break;

            /**
             * PayPal notify URL handler. Using for PayPal IPN.
             */
            case self::PAYPAL_ACTION_NOTIFY:
                if ( ! empty( $_POST ) && $this->validate_ipn() ) {
                    update_option( 'ipnvalidate_inner_listener', 1 );

                    $order = new Es_Order();

                    if ( ! empty( $_POST['recurring_payment_id'] ) && ( $order = $order->get_by_field( 'paypal_profile_id', $_POST['recurring_payment_id'] ) ) ) {
                        $status = ! empty( $_POST['initial_payment_status'] ) ? $_POST['initial_payment_status'] : $_POST['payment_status'];
                        $agent = $order->get_agent();

                        switch( $status ) {
                            case 'Completed':
                            case 'Cleared':
                                if ( es_subscription_is_user_has_subscription( $agent->getID(), $order->get_subscription()->getID() ) ) {
                                    es_subscription_activate_user_subscription( $agent->getID() );
                                    es_subscription_prolong_user_subscription( $order->get_subscription()->period, $agent->getID() );
                                } else {
                                    es_subscription_subscribe_user( $order->get_subscription()->getID(), $agent->getID() );
                                }
                                break;

                            case 'Denied':
                            case 'Refunded':
                            case 'Canceled_Reversal':
                            case 'Expired':
                            case 'Failed':
                            case 'Held':
                            case 'Hold':
                            case 'Returned':
                            case 'Reversed':
                            case 'Voided':
                            es_subscription_unactivate_user_subscription( $agent->getID() );
                                break;
                        }

                        $order->change_status( $status );

                    } else {
                        update_option( 'recurring_order_validate', 0 );
                    }
                }

                break;

            case self::PAYPAL_ACTION_CANCELED:
                $order = new Es_Order();
                if ( ! empty( $paypal_token ) && ( $order = $order->get_by_field( 'token', $paypal_token ) ) ) {
                    // Change order status to completed.
                    $order->change_status( 'Cancelled' );
                    $logger->set_message( __( 'Payment successfully canceled.', 'es-plugin' ), 'error' );
                }

                wp_redirect( $redirect_page ); exit;
                break;
        }
    }

    /**
     * Check for valid paypal.
     *
     * @return bool
     */
    public function valid_payment()
    {
        global $es_settings;
        return ! empty( $es_settings->payment_username ) && ! empty( $es_settings->payment_password ) && ! empty( $es_settings->payment_signature );
    }
}
