<?php

define( 'ES_PUBLISHING_AUTOMATIC', 'automatic' );
define( 'ES_RESTORE_CAPS', true );
define( 'ES_REMOVE_CAPS', false );
/**
 * Return all subscription periods.
 *
 * Duration is num of seconds in our period. 8600 = 24 * 60 * 60 - seconds in one day.
 * Date T - num of days in current year. Date L - return 1 If year is leap.
 *
 * @return array
 */
function es_get_subscription_periods() {
    return apply_filters( 'es_get_subscription_periods', array(
        'Week'  => array( 'title' => __( 'Week',  'es-plugin' ), 'duration' => 7 * 86400 ),
        'Month' => array( 'title' => __( 'Month', 'es-plugin' ), 'duration' => date( 't' ) * 86400 ),
        'Year'  => array( 'title' => __( 'Year',  'es-plugin' ), 'duration' => date( 'L' ) ? 366 * 86400 : 365 * 86400 ),
    ) );
}

/**
 * Check is user have subscription.
 *
 * @param $user_id
 * @return bool
 */
function es_subscription_is_user_has_subscription( $user_id = null, $subscription_id = null ) {
    $agent = es_get_agent( $user_id );
    if ( $subscription_id ) {
        return $agent->subscription_id == $subscription_id;
    }
    return (bool) $agent->subscription_expired;
}

/**
 * Change user capabilities for manage properties.
 *
 * @param null $user_id
 * @param bool $restore_caps
 * @param null $caps
 */
function es_subscription_change_user_caps( $user_id = null, $restore_caps = ES_RESTORE_CAPS, $caps = null ) {
    $user = es_get_agent( $user_id )->get_entity();

    $caps = $caps ? $caps : array( 'create_es_properties', 'publish_es_properties' );

    foreach ( $caps as $cap ) {
        if ( $restore_caps == ES_RESTORE_CAPS ) {
            if ( ! $user->has_cap(  $cap ) ) {
                $user->add_cap( $cap );
            }
        } else if ( $restore_caps == ES_REMOVE_CAPS ) {
            if ( $user->has_cap( $cap ) ) {
                $user->remove_cap( $cap );
            }
        }
    }
}

/**
 * Change role capabilities for manage properties.
 *
 * @param bool $restore_caps
 * @param string $role
 * @param null $caps
 */
function es_subscription_change_role_caps( $restore_caps = ES_RESTORE_CAPS, $role = null, $caps = null ) {
    $role = $role ? $role : Es_Agent::get_role_name();
    $role = get_role( $role );

    $caps = $caps ? $caps : array( 'create_es_properties', 'publish_es_properties' );

    foreach ( $caps as $cap ) {
        if ( $restore_caps == ES_RESTORE_CAPS ) {
            if ( ! $role->has_cap(  $cap ) ) {
                $role->add_cap( $cap );
            }
        } else if ( $restore_caps == ES_REMOVE_CAPS ) {
            if ( $role->has_cap( $cap ) ) {
                $role->remove_cap( $cap );
            }
        }
    }
}

/**
 * Activate user subscription.
 *
 * @param null $user_id
 */
function es_subscription_activate_user_subscription( $user_id = null ) {
    if ( es_subscription_is_user_has_subscription( $user_id ) ) {
        $agent = es_get_agent( $user_id );

        es_subscription_change_user_caps( $user_id, ES_RESTORE_CAPS );

        $agent->save_field_value( 'subscription_active', 1 );
    }
}

/**
 * Unactivate user subscription.
 *
 * @param null $user_id
 */
function es_subscription_unactivate_user_subscription( $user_id = null ) {

    if ( es_subscription_is_user_has_subscription( $user_id ) ) {

        /** @var Es_Agent $agent */
        $agent = es_get_agent( $user_id );

        es_subscription_change_user_caps( $user_id, ES_REMOVE_CAPS );

        $agent->save_field_value( 'subscription_active', 0 );

        es_subscription_update_user_properties_status( $user_id );

        do_action( 'es_subscription_expired', $user_id );
    }
}

/**
 * Return list of all subscriptions.
 *
 * @return array
 */
function es_get_subscriptions() {
    return get_posts(array(
        'post_type' => 'es_subscription',
        'posts_per_page' => -1, // All subscriptions
        'post_status' => 'publish',
        'order' => 'ASC',
    ));
}

/**
 * Return initialized object of payment method.
 *
 * @return Es_Subscription_Payment_Method
 */
function es_subscription_get_payment_method_class() {
    global $es_settings;

    $classes = es_subscription_get_payment_methods();

    return ! empty( $es_settings->payment_method ) ? new $classes[ $es_settings->payment_method ]['class'] : null;
}

/**
 * Return list of available payment methods.
 *
 * @return array
 */
function es_subscription_get_payment_methods() {
    return apply_filters( 'es_get_subscription_payment_methods', array(
        'paypal_subscription' => array(
            'title' => __( 'PayPal Subscription', 'es-plugin' ),
            'class' => 'Es_PayPal_Subscription_Payment_Method'
        ),
    ) );
}

/**
 * Check if subscription is enabled.
 *
 * @return bool
 */
function es_subscription_is_enabled() {
    global $es_settings;
    return (bool) $es_settings->enabled_subscription;
}

/**
 * Update agent subscription expiration date.
 *
 * @param int|string $period
 *    Can be timestamp of key of the @see es_get_subscription_periods()
 * @param null $user_id
 *    User ID
 *
 * @return bool
 */
function es_subscription_prolong_user_subscription( $period, $user_id = null ) {
    $agent = es_get_agent( $user_id );

    if ( $agent->get_entity() ) {
        $expired = $agent->subscription_expired;

        if ( is_int( $period ) ) {
            $period = $period + $expired;
        } else {
            $periods = es_get_subscription_periods();
            $period = ! empty( $periods[ $period ]['duration'] ) ? $periods[ $period ]['duration'] + $expired : $expired;
        }

        $agent->save_field_value( 'subscription_expired', time() + $period + 7200 );
        es_subscription_activate_user_subscription( $user_id );
    }

    return false;
}

/**
 * Subscribe user on subscription.
 *
 * @param $subscription_id
 * @param null $user_id
 */
function es_subscription_subscribe_user( $subscription_id, $user_id = null ) {

	/** @var Es_Agent $agent */
	$agent = es_get_agent( $user_id );

	/** @var WP_User $user */
	$user = $agent->get_entity();

	/** @var Es_Subscription $subscription */
	$subscription = es_get_subscription( $subscription_id );

	if ( $user && $subscription->get_entity() ) {
		$periods = es_get_subscription_periods();

		$agent->save_field_value( 'subscription_property_qty', $subscription->property_qty );
		$agent->save_field_value( 'subscription_featured_qty', $subscription->featured_qty );
		$agent->save_field_value( 'subscription_id', $subscription->get_entity()->ID );
		$agent->save_field_value( 'subscription_published_posts', 0 );
		$agent->save_field_value( 'subscription_featured_posts', 0 );
		$agent->save_field_value( 'subscription_expired', time() + $periods[ $subscription->period ]['duration'] + 7200 );

        es_subscription_activate_user_subscription( $user_id );
	}

	do_action( 'es_subscription_after_subscribe_user', $agent, $subscription );
}

/**
 * Unsubscribe user from subscription.
 *
 * @param $user_id
 */
function es_subscription_unsubscribe_user( $user_id ) {

	/** @var Es_Agent $agent */
	$agent = es_get_agent( $user_id );

	/** @var WP_User $user */
	$user = $agent->get_entity();

	if ( $user ) {
        es_subscription_unactivate_user_subscription( $user_id );

        $agent->delete_field_value( 'subscription_property_qty' );
        $agent->delete_field_value( 'subscription_featured_qty'  );
        $agent->delete_field_value( 'subscription_id'  );
        $agent->delete_field_value( 'subscription_published_posts' );
        $agent->delete_field_value( 'subscription_featured_posts' );
        $agent->delete_field_value( 'subscription_expired' );
        $agent->delete_field_value( 'subscription_active' );

	}

	do_action( 'es_subscription_after_unsubscribe_user', $agent );
}

/**
 * Update user properties status.
 *
 * @param $users_id
 */
function es_subscription_update_user_properties_status( $users_id = null, $status = 'draft' ) {
    global $wpdb;
    $users_id = $users_id ? $users_id : get_current_user_id();

    $wpdb->update( $wpdb->posts, array(
        'post_status' => $status,
    ), array(
        'post_author' => $users_id,
        'post_type' => Es_Property::get_post_type_name(),
    ) );
}

/**
 * Check is user subscription expired.
 *
 * @param $user_id
 * @return bool
 */
function es_subscription_is_user_subscription_expired( $user_id = null ) {
    $agent = es_get_agent( $user_id );

    if ( time() < $agent->subscription_expired ) {
        return false;
    }

    return true;
}

/**
 * Check if user subscription is active.
 *
 * @param $user_id
 * @return bool
 */
function es_subscription_is_user_subscription_active( $user_id ) {
    $agent = es_get_agent( $user_id );
    return $agent->subscription_active;
}

/**
 * Return user subscription object.
 *
 * @param null|integer $user_id
 * @return Es_Subscription|null
 */
function es_subscription_get_user_subscription( $user_id = null ) {
	$agent = es_get_agent( $user_id );

	return ! empty( $agent->subscription_id ) ?
		es_get_subscription( $agent->subscription_id ) : null;
}

/**
 * Return subscription object by ID.
 *
 * @param $subscription_id
 * @return Es_Subscription
 */
function es_get_subscription( $subscription_id ) {
	return apply_filters( 'es_get_subscription', new Es_Subscription( $subscription_id ) );
}

/**
 * Return PayPal response page.
 *
 * @param bool $single
 * @return array|WP_Post
 */
function es_subscription_is_available_paypal_response_page( $single = true ) {
	$posts = get_pages( array(
		'meta_key' => 'es_subscription_paypal_response',
		'meta_value' => 1,
	) );

	if ( $single && $posts ) {
		$posts = $posts[0];
	}

	return $posts;
}

/**
 * Return subscription table page.
 *
 * @param bool $single
 * @return array|false
 */
function es_subscription_is_available_subscription_table_page( $single = true ) {
    $posts = get_pages( array(
        'meta_key' => 'es_subscription_table_page',
        'meta_value' => 1,
    ) );

    if ( $single && $posts ) {
        $posts = $posts[0];
    }

    return $posts;
}

/**
 * Return current paypal response page object.
 *
 * @return null
 */
function es_subscription_get_current_paypal_response_page() {
	global $es_settings;

	$id = $es_settings->payment_response_page;

	if ( $id ) {
		return get_post( $id );
	}

	return null;
}

/**
 * Return all pages.
 *
 * @return array|false
 */
function es_subscription_get_pages() {
	$posts = get_pages( array(
		'nopaging' => true,
		'posts_per_page' => -1,
        'post_type' => 'page',
	) );

	return $posts;
}

/**
 * Create PayPal Response page.
 *
 * @return bool
 */
function es_subscription_create_paypal_response_page() {
    if ( ! es_subscription_is_available_paypal_response_page() ) {
        $post = wp_insert_post( array(
            'post_content' => '[es_subscription_paypal_response_page]',
            'post_content_filtered' => '',
            'post_title' => 'PayPal Success',
            'post_status' => 'publish',
            'post_type' => 'page',
        ) );

        if ( ! $post instanceof WP_Error && ! empty( $post ) ) {
            update_post_meta( $post, 'es_subscription_paypal_response', 1 );

            update_option( 'es_payment_response_page', $post );
        }
    }

    return false;
}

/**
 * Create PayPal Response page.
 *
 * @return bool
 */
function es_subscription_create_subscription_table_page() {
    if ( ! es_subscription_is_available_subscription_table_page() ) {
        $post = wp_insert_post( array(
            'post_content' => '[es_subscription_table]',
            'post_content_filtered' => '',
            'post_title' => 'PayPal Subscriptions',
            'post_status' => 'publish',
            'post_type' => 'page',
        ) );

        if ( ! $post instanceof WP_Error && ! empty( $post ) ) {
            update_post_meta( $post, 'es_subscription_table_page', 1 );

            update_option( 'es_subscription_table_page', $post );
        }
    }

    return false;
}

/**
 * Return subscription checkout page.
 *
 * @return array|null|WP_Post
 */
function es_subscription_get_checkout_page()
{
    global $es_settings;

    return ! empty( $es_settings->subscription_checkout_page_id ) ?
        get_post( $es_settings->subscription_checkout_page_id ) : null;
}

/**
 * Render agent system messages about subscription status.
 *
 * @return void
 */
function es_subscription_agent_messages() {
    // If subscription is emabled on the site.
    if ( es_subscription_is_enabled() && is_user_logged_in() ) {

        if ( current_user_can( 'administrator' ) ) return;

        // Agent object.
        $agent = es_get_agent( get_current_user_id() );
        // Num of published posts.
        $already_published = $agent->subscription_published_posts;
        // Limit for publish
        $limit_published = $agent->subscription_property_qty;
        // Subscription checkout page.
        $checkout = $post = es_subscription_get_checkout_page();

        $label = __( 'subscription plan', 'es-plugin' );

        // If user has subscription (or had subscription before).
        if ( es_subscription_is_user_has_subscription() ) {

            if ( $agent->subscription_expired < time() ) {
                $msg = sprintf(
                    __( 'Your subscription is expired, please update your %s.', 'es-plugin' ),
                    $checkout ? '<a href="' . get_the_permalink( $checkout ) .'">' . $label . '</a>' : $label
                );
            } else if ( $limit_published <= $already_published || ! current_user_can( 'create_es_properties' ) )  {
                $msg = sprintf(
                    __( "You subscription plan doesn't allow to publish more than %s listings, please update your %s or unpublish some properties.", 'es-plugin' ),
                    $limit_published,
                    $checkout ? '<a href="' . get_the_permalink( $checkout ) .'">' . $label . '</a>' : $label
                );
            }
        } else {
            $msg = sprintf(
                __( 'Your have no subscription to publish new properties, please update your %s.', 'es-plugin' ),
                $checkout ? '<a href="' . get_the_permalink( $checkout ) .'">' . $label . '</a>' : $label
            );
        }

        if ( ! empty( $msg ) ) es_render_message( $msg, 'warning' );
    }
}

add_action( 'save_post', 'es_agent_change_property_qty' );

/**
 * @param $post
 */
function es_agent_change_property_qty( $post ) {
	if ( is_int( $post ) ) {
		$post = get_post( $post );
		$agent = es_get_agent( $post->post_author );

		if ( $agent->is_agent() ) {
			global $wpdb;

			$count_posts = $wpdb->get_var( "SELECT COUNT(*) 
				FROM {$wpdb->posts} 
				WHERE post_author='{$agent->getID()}' AND post_status='publish' AND post_type='" . Es_Property::get_post_type_name() . "'" );

			$agent->save_field_value( 'property_qty', $count_posts );
		}
	}
}